SystemOrganization addCategory: #'OmniBrowser-Actors'!
SystemOrganization addCategory: #'OmniBrowser-Announcements'!
SystemOrganization addCategory: #'OmniBrowser-Commands'!
SystemOrganization addCategory: #'OmniBrowser-Kernel'!
SystemOrganization addCategory: #'OmniBrowser-Morphic'!
SystemOrganization addCategory: #'OmniBrowser-Nodes'!
SystemOrganization addCategory: #'OmniBrowser-Notifications'!
SystemOrganization addCategory: #'OmniBrowser-Panels'!
SystemOrganization addCategory: #'OmniBrowser-Utilities'!

Notification subclass: #OBAnnouncerRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

Notification subclass: #OBInteractionRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBInteractionRequest commentStamp: 'cwp 12/7/2004 00:13' prior: 0!
OBInteractionRequest is an abstract superclass for notifications that request some interaction with the user. It's useful for catching such notifications in an exception handler, while allowing other notifications to operate normally.!

OBInteractionRequest subclass: #OBBrowseRequest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBBrowseRequest commentStamp: 'cwp 12/7/2004 00:10' prior: 0!
This notification is raised whenever a browser needs to be opened. The default action is to open a SystemWindow in Morphic, but it can be caught in situations where that's not appropriate. The OB test suite uses this extensively. !

!OBBrowseRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 18:45'!
signal: aBrowser
	^ (self new browser: aBrowser) signal! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser
	^ browser! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser: aBrowser
	browser _ aBrowser! !

!OBBrowseRequest methodsFor: 'dispatching' stamp: 'cwp 5/25/2007 22:51'!
handleWith: anObject
	^ anObject handleBrowseRequest: self! !

!OBBrowseRequest methodsFor: 'testing' stamp: 'cwp 10/17/2004 19:32'!
isBrowseRequest
	^ true! !

OBInteractionRequest subclass: #OBChoiceRequest
	instanceVariableNames: 'prompt labels values lines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBChoiceRequest commentStamp: 'cwp 3/5/2004 12:26' prior: 0!
This notification is used to ask the user for to choose from a list of alternatives. Its defaultAction is to open a PopUpMenu. Test cases an intercept the notification and respond programmatically.

prompt	- A string describing the choice the user is asked to make.
labels	- A list of strings describing the alternatives.
values	- When the user chooses an alternative, the corresponding item from this list is returned!

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:07'!
labels: anArray
	^ self
		prompt: nil
		labels: anArray
		values: anArray
		lines: #()! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:08'!
labels: labelArray lines: lineArray
	^ self
		prompt: nil
		labels: labelArray
		values: labelArray
		lines: lineArray! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:06'!
prompt: aString labels: labelArray values: valueArray
	^ self
		prompt: aString
		labels: labelArray
		 values: valueArray
		lines: #()! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:05'!
prompt: aString labels: labelArray values: valueArray lines: lineArray
	^ (self new 
		setPrompt: aString 
		labels: labelArray 
		values: valueArray 
		lines: lineArray) 
			signal! !

!OBChoiceRequest methodsFor: 'dispatching' stamp: 'cwp 6/1/2007 16:38'!
handleWith: anObject
	^ anObject handleChoiceRequest: self! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/8/2006 23:45'!
labels
	^ labels! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/9/2006 00:11'!
lines
	^ lines! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/8/2006 23:45'!
prompt
	^ prompt! !

!OBChoiceRequest methodsFor: 'signaling' stamp: 'cwp 10/31/2006 20:38'!
select: aString
	| index |
	index _ labels indexOf: aString.
	self resume: (values at: index ifAbsent: [nil])! !

!OBChoiceRequest methodsFor: 'initializing' stamp: 'cwp 7/9/2006 00:05'!
setPrompt: aString labels: labelArray values: valueArray lines: lineArray
	prompt _ aString.
	labels _ labelArray.
	values _ valueArray.
	lines _ lineArray.! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/8/2006 23:45'!
values
	^ values! !

OBInteractionRequest subclass: #OBConfirmationRequest
	instanceVariableNames: 'prompt confirm cancel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBConfirmationRequest commentStamp: 'cwp 3/5/2004 12:30' prior: 0!
This notification is used to ask the user to confirm some kind potentially dangerous operation. Its default action is to open a PopUpMenu.

iVars:

prompt		- a string describing the situation the user is asked to confirm
confirm		- a string describing the action that will be taken if the user confirms
cancel		- a string describing the action that will be taken if the user does not confirm!

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:09'!
newPrompt: prompt confirm: confirm cancel: cancel
	^ self new setPrompt: prompt confirm: confirm cancel: cancel! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:13'!
prompt: prompt
	^ self prompt: prompt confirm: 'Ok'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:12'!
prompt: prompt confirm: confirm
	^ self prompt: prompt confirm: confirm cancel: 'Cancel'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:02'!
prompt: prompt confirm: confirm cancel: cancel
	^ (self newPrompt: prompt confirm: confirm cancel: cancel) signal! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 6/1/2007 16:58'!
cancel
	^ self resume: false! !

!OBConfirmationRequest methodsFor: 'accessing' stamp: 'cwp 6/1/2007 16:46'!
cancelChoice
	^ cancel! !

!OBConfirmationRequest methodsFor: 'dispatching' stamp: 'cwp 6/1/2007 16:40'!
handleWith: anObject
	^ anObject handleConfirmationRequest: self! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
ok
	self resume: true! !

!OBConfirmationRequest methodsFor: 'accessing' stamp: 'cwp 6/1/2007 16:46'!
okChoice
	^ confirm! !

!OBConfirmationRequest methodsFor: 'accessing' stamp: 'cwp 6/1/2007 16:44'!
prompt
	^ prompt! !

!OBConfirmationRequest methodsFor: 'initalizing' stamp: 'cwp 2/28/2004 12:00'!
setPrompt: promptString confirm: confirmString cancel: cancelString
	prompt _ promptString.
	confirm _ confirmString.
	cancel _ cancelString! !

!OBInteractionRequest methodsFor: 'as yet unclassified' stamp: 'cwp 6/1/2007 16:42'!
defaultAction
	^ self handleWith: OBPlatform current! !

!OBInteractionRequest methodsFor: 'dispatching' stamp: 'PDC 6/26/2007 22:00'!
handleWith: anObject
	self subclassResponsibility ! !

!OBInteractionRequest methodsFor: 'testing' stamp: 'cwp 10/17/2004 19:31'!
isBrowseRequest
	^ false! !

OBInteractionRequest subclass: #OBTextRequest
	instanceVariableNames: 'prompt template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBTextRequest commentStamp: 'cwp 3/5/2004 12:35' prior: 0!
This notification is used to ask the user to supply a short piece of text. Its defaultAction is to open a FillInTheBlank.

iVars:

prompt		- a string describing the text the user is asked to supply
template	- a default reply !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'PDC 6/26/2007 22:16'!
prompt: aString 
	^ self new prompt: aString;
		 template: '';
		 yourself! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'PDC 6/26/2007 22:16'!
prompt: promptString template: templateString 
	^ (self new prompt: promptString;
		 template: templateString;
		 yourself) signal! !

!OBTextRequest methodsFor: 'dispatching' stamp: 'cwp 6/1/2007 16:40'!
handleWith: anObject
	^ anObject handleTextRequest: self! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:24'!
prompt
	^ prompt! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:23'!
prompt: aString
	prompt _ aString! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:00'!
template
	^ template! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 20:59'!
template: aString
	template _ aString! !

!Text methodsFor: '*omnibrowser' stamp: 'dr 3/6/2007 16:26'!
colorAt: characterIndex
    | textColor | 
    textColor _ (self attributesAt: characterIndex) detect: [:att | att isMemberOf: TextColor] ifNone: [TextColor black].
    ^textColor color! !

!MessageSend methodsFor: '*omnibrowser-evaluating' stamp: 'cwp 6/24/2006 18:19'!
valueWithPossibleArgs: anArray
	^ receiver 
		perform: selector 
		withArguments: (self collectArguments: anArray)! !

!MenuIcons class methodsFor: '*omnibrowser-services' stamp: 'cwp 7/10/2006 22:32'!
tryIcons: anArray
	| selector |
	selector := anArray detect: [:ea | self respondsTo: ea] ifNone: [self blankIcon].
	^ MenuIcons perform: selector! !

Object subclass: #OBAnnouncement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBAnnouncement subclass: #OBAboutToChange
	instanceVariableNames: 'veto'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBAboutToChange methodsFor: 'vetos' stamp: 'PDC 6/26/2007 20:35'!
initialize
	super initialize.
	veto := false! !

!OBAboutToChange methodsFor: 'vetos' stamp: 'PDC 6/26/2007 20:32'!
isVetoed
	^ veto ifNil: [false]! !

!OBAboutToChange methodsFor: 'vetos' stamp: 'PDC 6/26/2007 20:32'!
veto
	veto := true! !

OBAnnouncement subclass: #OBAboutToChangeSilently
	instanceVariableNames: 'veto'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBAboutToChangeSilently methodsFor: 'vetos' stamp: 'PDC 6/26/2007 20:34'!
initialize
	super initialize.
	veto := false! !

!OBAboutToChangeSilently methodsFor: 'vetos' stamp: 'PDC 6/26/2007 20:33'!
isVetoed
	^ veto ifNil: [false]! !

!OBAboutToChangeSilently methodsFor: 'vetos' stamp: 'PDC 6/26/2007 20:34'!
veto
	veto := true! !

OBAnnouncement subclass: #OBChildrenChanged
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBChildrenChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:49'!
node: aNode
	^ self new node: aNode! !

!OBChildrenChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:49'!
node
	^ node! !

!OBChildrenChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:49'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBCommandScan
	instanceVariableNames: 'factories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBCommandScan methodsFor: 'accessing' stamp: 'cwp 11/2/2006 00:35'!
addFactory: aCommandFactory 
	factories add: aCommandFactory! !

!OBCommandScan methodsFor: 'accessing' stamp: 'cwp 11/2/2006 00:30'!
commandsOn: aNode for: aRequestor
	^ factories collect: [:ea | ea on: aNode for: aRequestor]! !

!OBCommandScan methodsFor: 'user interface' stamp: 'cwp 10/15/2006 00:33'!
groupCommands: commands satisfying: aBlock
	| groups |
	groups _ (commands select: aBlock)
				groupBy: [:svc | svc group]
				having: [:group | group isEmpty not].
	groups _ groups collect: [:ea | ea asArray sort: [:a :b | a label <= b label]].
	^ groups asArray sort: [:a :b | a anyOne group <= b anyOne group].
	! !

!OBCommandScan methodsFor: 'initialize-release' stamp: 'cwp 11/2/2006 00:30'!
initialize
	factories _ OrderedCollection new! !

!OBCommandScan methodsFor: 'user interface' stamp: 'lr 4/25/2007 09:15'!
populateMenu: aMenu withNode: aNode forRequestor: aRequestor 
	^ self 
		populateMenu: aMenu
		withNode: aNode
		forRequestor: aRequestor
		filter: [:svc | svc isActive]! !

!OBCommandScan methodsFor: 'user interface' stamp: 'cwp 10/15/2006 01:16'!
populateMenu: aMenu withNode: aNode forRequestor: aRequestor filter: aBlock

	| commands |
	commands _ self commandsOn: aNode for: aRequestor.
	(self groupCommands: commands satisfying: aBlock) do: 
		[:group |
		aMenu items isEmpty ifFalse: [aMenu addLine]. 
		group do: [:svc | svc addItemToMenu: aMenu]]
			! !

!OBCommandScan methodsFor: 'user interface' stamp: 'cwp 10/15/2006 14:06'!
processKeystroke: aCharacter withNode: aNode for: aRequestor
	| commands |
	commands _ self commandsOn: aNode for: aRequestor. 
	(commands select: [:cmd | cmd keystroke == aCharacter]) do:
		[:cmd |
		(cmd isActive and: [cmd isEnabled])
			ifTrue: [cmd execute]]! !

OBCommandScan subclass: #OBNodeCommandScan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBCommandScan subclass: #OBTextCommandScan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBAnnouncement subclass: #OBDefinitionChanged
	instanceVariableNames: 'definition node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBDefinitionChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:28'!
definition: aDefinition
	^ self new
		definition: aDefinition;
		yourself! !

!OBDefinitionChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:27'!
node: aNode definition: aDefinition
	^ self new
		node: aNode;
		definition: aDefinition;
		yourself! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:25'!
definition
	^ definition! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:26'!
definition: aDefinition
	definition _ aDefinition! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:47'!
node
	^ node! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:47'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBNodeChanged
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBNodeChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:17'!
node: aNode
	^ self new node: aNode! !

!OBNodeChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:17'!
node
	^ node! !

!OBNodeChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:17'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBNodeCreated
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBNodeCreated class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 01:09'!
node: aNode
	^ self new node: aNode! !

!OBNodeCreated methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 00:53'!
node
	^ node! !

!OBNodeCreated methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 00:53'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBNodeDeleted
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBNodeDeleted class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:56'!
node: aNode
	^ self new node: aNode! !

!OBNodeDeleted methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:56'!
node
	^ node! !

!OBNodeDeleted methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:56'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBRefreshRequired
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBAnnouncement subclass: #OBSelectingNode
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBSelectingNode class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 13:58'!
node: aNode
	^ self new node: aNode! !

!OBSelectingNode methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 13:58'!
node
	^ node! !

!OBSelectingNode methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 13:58'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBSelectionChanged
	instanceVariableNames: 'node column deselectedNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBSelectionChanged class methodsFor: 'instance creation' stamp: 'cwp 4/19/2006 00:51'!
column: aColumn 
	^ self new column: aColumn! !

!OBSelectionChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 4/19/2006 00:31'!
node: aNode
	^ self new node: aNode! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 4/19/2006 00:53'!
column
	^ column! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 4/19/2006 00:52'!
column: aColumn
	column _ aColumn! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'dr 5/21/2007 16:08'!
deselectedNode
	^deselectedNode! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'dr 5/21/2007 16:08'!
deselectedNode: aNode
	deselectedNode := aNode! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 6/4/2006 12:19'!
node
	^ column 
		ifNil: [node]
		ifNotNil: [column selectedNode]! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 6/4/2006 12:20'!
node: aNode
	node _ aNode! !

Object subclass: #OBAnnouncer
	instanceVariableNames: 'subscriptions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBAnnouncer class methodsFor: 'instance creation' stamp: 'cwp 6/4/2006 00:55'!
current
	^ OBAnnouncerRequest signal! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'dkh 6/4/2007 09:23'!
announce: anObject

	| ann |
	ann _ anObject asAnnouncement.
	subscriptions keysAndValuesDo:
		[:class :action |
		(ann isKindOf: class) ifTrue: [action valueWithArguments: (Array with: ann)]].
	^ ann! !

!OBAnnouncer methodsFor: 'initialize-release' stamp: 'cwp 4/17/2006 11:51'!
initialize
	subscriptions _ IdentityDictionary new.! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/3/2006 21:21'!
observe: aClass do: aValuable 
	| actions |
	actions _ subscriptions at: aClass ifAbsent: [ActionSequence new].
	subscriptions at: aClass put: (actions copyWith: aValuable).! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/3/2006 23:31'!
observe: aClass send: aSelector to: anObject
	self
		observe: aClass
		do: (MessageSend receiver: anObject selector: aSelector)! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/10/2006 00:40'!
unsubscribe: anObject
	subscriptions keysAndValuesDo:
		[:class :actions |
		subscriptions at: class put: (actions reject: [:ea | ea receiver == anObject])].
	subscriptions keysAndValuesRemove: [:key :value | value isEmpty]! !

Object subclass: #OBBrowser
	instanceVariableNames: 'panels announcer cmdFactories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBrowser commentStamp: 'cwp 1/7/2005 23:17' prior: 0!
OBBrowser is the core of OmniBrower. It's the root object for each browser and the model for the SystemWindows which display them. Its main responsibily is managing panels, particularly passing update messages between them.

On the class side, OBBrowser provides some default settings for creating and opening browsers. Subclasses can override these settings to acheive customized behavior.

iVars:

panels	- A collection of objects which manage submorphs of the browser's SystemWindow.

cVars:

MetaGraphs - A dictionary matching names to metagraphs!

!OBBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:10'!
defaultMetaNode
	self subclassResponsibility! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/6/2004 23:20'!
defaultRootNode
	self subclassResponsibility! !

!OBBrowser class methodsFor: 'configuration' stamp: 'dr 5/29/2007 11:15'!
definitionPanel
	^OBDefinitionPanel new! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
maxPanes
	^ self paneCount! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:12'!
metaNode: metaNode node: rootNode 
	^ self metaNode: metaNode root: rootNode selection: nil panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 01:16'!
metaNode: metaNode root: rootNode selection: selectedNode
	^ self metaNode: metaNode root: rootNode selection: selectedNode panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'dr 7/16/2007 11:24'!
metaNode: metaNode root: rootNode selection: selectedNode panels: panels
	| browser |
	browser _ self basicNew initialize.
	panels do: [:ea | browser addPanel: ea].
	browser setMetaNode: metaNode node: rootNode.
	selectedNode ifNotNil: [browser jumpTo: selectedNode].
	^ browser! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:00'!
minPanes
	^ self paneCount! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 16:55'!
navigationPanel
	^ OBColumnPanel minPanes: self minPanes maxPanes: self maxPanes! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/16/2005 23:25'!
new
	^ self 
		metaNode: self defaultMetaNode
		root: self defaultRootNode
		selection: nil
		panels: self panels! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/6/2004 23:22'!
open
	^ self new open! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 1/22/2007 23:21'!
openRoot: aNode
	^ (self root: aNode) open! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/5/2004 17:03'!
openRoot: rootNode selection: selectedNode
	^ (self root: rootNode selection: selectedNode) open! !

!OBBrowser class methodsFor: 'configuration' stamp: 'dr 7/2/2007 16:21'!
optionalButtonPanel
	^Preferences optionalButtons ifTrue: [OBFixedButtonPanel new]! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:07'!
optionalButtons
	^ #()! !

!OBBrowser class methodsFor: 'configuration' stamp: 'dr 4/24/2007 15:37'!
paneCount
	^ 4! !

!OBBrowser class methodsFor: 'configuration' stamp: 'dkh 5/31/2007 13:45'!
panels
	^ (Array with: self navigationPanel with: self optionalButtonPanel with: self definitionPanel)
		reject: [:ea | ea isNil]! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 15:26'!
root: aNode 
	^ self root: aNode selection: nil! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/16/2005 23:25'!
root: rootNode selection: selectedNode
	^ self metaNode: self defaultMetaNode root: rootNode selection: selectedNode panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/6/2004 22:04'!
selection: selectedNode
	^ self root: self defaultRootNode selection: selectedNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:32'!
title
	^ 'OmniBrowser'! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 13:02'!
titleForRoot: aNode
	^ self title! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 6/4/2006 00:36'!
addPanel: aPanel
	panels add: aPanel.
	aPanel browser: self.
! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 10/4/2006 10:06'!
announce: anObject
	^ self announcer announce: anObject! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 4/17/2006 15:42'!
announcer
	^ announcer! !

!OBBrowser methodsFor: 'building' stamp: 'cwp 3/18/2007 20:46'!
buildGroup: aCollection on: aBuilder
	^ aBuilder
		panelGroup: aCollection
		with: [aCollection do: [:ea | ea buildOn: aBuilder]]! !

!OBBrowser methodsFor: 'building' stamp: 'cwp 3/18/2007 20:47'!
buildOn: aBuilder
	
	| group |
	^ aBuilder browser: self with: 
		[group _ OrderedCollection with: panels first.
		panels allButFirst do: 
			[:panel | 
			group last vResizing = #spaceFill ifTrue:
				[self buildGroup: group on: aBuilder.
				group _ OrderedCollection new].
			group add: panel].
		self buildGroup: group on: aBuilder]! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 3/11/2007 18:29'!
cmdFactories
	^ cmdFactories! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentNode
	^self navigationPanel currentNode! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentOrRootNode
	^self navigationPanel currentOrRootNode! !

!OBBrowser methodsFor: 'building' stamp: 'cwp 11/27/2004 22:21'!
defaultBackgroundColor
	^ Color gray veryMuchLighter! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 8/31/2004 13:04'!
defaultLabel
	^ self class titleForRoot: self root! !

!OBBrowser methodsFor: 'accessing' stamp: 'dr 5/24/2007 12:21'!
definitionPanel
	^ panels 
		detect: [:ea | ea isDefinition] 
		ifNone: [self error: 'No definition panel configured']! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:29'!
dontTranscribe
	self announcer unsubscribe: self.
	self subscribe.! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 11/2/2006 00:26'!
initialize
	panels _ OrderedCollection new.
	announcer _ OBAnnouncer new.
	cmdFactories _ #().
	self subscribe.! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 11/2/2006 00:26'!
initializeCommands
	cmdFactories := Array streamContents: 
					[:stream | 
					(self class allSelectors select: [:ea | ea beginsWith: 'cmd']) do: 
							[:selector | 
							| result |
							result := self perform: selector.
							result isCollection 
								ifTrue: [stream nextPutAll: result]
								ifFalse: [stream nextPut: result]]]! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpTo: aNode 
	self navigationPanel jumpTo: aNode! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpToRoot
	^self navigationPanel jumpToRoot! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 11/20/2004 21:05'!
labelString
	| label |
	label := self navigationPanel labelString.
	^label 
		ifNil: [self defaultLabel]
		ifNotNil: [self defaultLabel , ': ' , label]! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 12/5/2004 16:25'!
navigationPanel
	^ panels 
		detect: [:ea | ea isNavigation] 
		ifNone: [self error: 'No navigation panel configured']! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 4/17/2006 19:52'!
okToChange
	^ (self announcer announce: OBAboutToChange) isVetoed not! !

!OBBrowser methodsFor: 'opening' stamp: 'cwp 10/17/2004 21:31'!
open
	^ OBBrowseRequest signal: self! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:53'!
panels
	^ panels! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 4/24/2006 15:10'!
relabel: ann
	self changed: #relabel.
! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 9/17/2005 17:48'!
root
	^self navigationPanel root! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/2/2006 00:35'!
scanNodeCommands: ann 
	(cmdFactories select: [:ea | ea takesNodes]) 
		do: [:ea | ann addFactory: ea]! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/2/2006 00:35'!
scanTextCommands: ann 
	(cmdFactories select: [:ea | ea takesText]) do: [:ea | ann addFactory: ea]! !

!OBBrowser methodsFor: 'accessing' stamp: 'dvf 9/5/2005 17:30'!
selectionPath
	^self navigationPanel selectionPath! !

!OBBrowser methodsFor: 'initializing' stamp: 'dr 4/17/2007 15:38'!
setMetaNode: aMetaNode node: aNode 
     "aNode browser: self."
	self navigationPanel setMetaNode: aMetaNode node: aNode.
	self initializeCommands! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 6/4/2006 18:50'!
signalRefresh
	self announcer announce: OBRefreshRequired! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/1/2006 23:46'!
subscribe
	(self announcer)
		observe: OBSelectionChanged
			send: #relabel:
			to: self;
		observe: OBNodeCommandScan
			send: #scanNodeCommands:
			to: self;
		observe: OBTextCommandScan
			send: #scanTextCommands:
			to: self! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 6/6/2006 00:26'!
transcribe
	self announcer observe: OBAnnouncement do: [:ann | Transcript cr; show: ann].! !

Object subclass: #OBBuilder
	instanceVariableNames: 'current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBuilder class methodsFor: 'as yet unclassified' stamp: 'cwp 3/18/2007 20:47'!
build: aModel
	| builder |
	builder _ self new.
	^ aModel buildOn: builder.
	! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:25'!
browser: aBrowser with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:25'!
column: aColumn with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:26'!
columnPanel: aPanel with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'accessing' stamp: 'PDC 6/26/2007 21:34'!
current: aWidget
	current := aWidget! !

!OBBuilder methodsFor: 'accessing' stamp: 'PDC 6/26/2007 21:29'!
current: anObject do: aBlock 
	| saved |
	
	saved := current.
	current := anObject.
	[aBlock value] 
			ensure: [current := saved].
	^ anObject! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:26'!
definitionPanel: aPanel with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/18/2007 22:20'!
fixedButtonPanel: aPanel with: aBlock
	self subclassResponsibility ! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:27'!
modalFilter: aFilter with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/18/2007 20:08'!
panelGroup: aCollection with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'accessing' stamp: 'cwp 3/13/2007 21:23'!
root
	^ current! !

Object subclass: #OBColumn
	instanceVariableNames: 'children panel parent selection switch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBColumn commentStamp: 'cwp 1/7/2005 23:19' prior: 0!
An OBColumn manages a list of nodes, which it displays in a PluggableListMorph in the pane scroller at the top of the browser. All instances of OBColumn belong to an OBColumnPanel. It's main responsibility is keeping its list - and those of its neighbours - up to date. Each column has a MetaNode, which provides the list contents. It uses a filter to meditate between its self and the MetaNode.

iVars:

panel		- the panel which owns the column
filter		- the filter which manages the column's MetaNode.
parent		- the node selected in the column to the left of this column
children 	- the nodes which make up this column's list
selection 	- the index of the node selected by the user!

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 9/18/2005 13:26'!
inPanel: aBrowser
	^ self new setPanel: aBrowser! !

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 9/18/2005 13:25'!
inPanel: aPanel metaNode: aMetaNode node: aNode
	^ self new
			setPanel: aPanel
			metaNode: aMetaNode
			node: aNode! !

!OBColumn methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:15'!
addCommandsToMenu: aMenu 
	| scan |
	scan := self announce: OBNodeCommandScan.
	scan 
		populateMenu: aMenu
		withNode: self parent
		forRequestor: self.
	self hasSelection 
		ifTrue: 
			[scan 
				populateMenu: aMenu
				withNode: self selectedNode
				forRequestor: self]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 10/13/2006 09:05'!
announce: anAnnouncement 
	^ self browser announce: anAnnouncement! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 4/17/2006 15:52'!
announcer
	^ panel announcer! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:49'!
basicNext
	^panel columnAfter: self! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/18/2007 20:35'!
basicParent: aNode
	parent _ aNode.
	self switchFilter: parent metaNode switchFilter.
	self getChildren.! !

!OBColumn methodsFor: 'accessing' stamp: 'rw 6/10/2006 14:48'!
browser

	^panel browser! !

!OBColumn methodsFor: 'building' stamp: 'dr 6/14/2007 11:06'!
buildOn: aBuilder
	^aBuilder 
		column: self 
		with: [switch buildOn: aBuilder]! !

!OBColumn methodsFor: 'updating' stamp: 'dr 6/7/2007 16:26'!
childrenChanged: announcement
	(self parent = announcement node) ifTrue:
		[self refresh]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 5/18/2007 20:24'!
clear
	self switchFilter: nil.
	children := #().
	self clearSelection.
	self changed: #list.
	self changed: #selection.
! !

!OBColumn methodsFor: 'selecting' stamp: 'dr 7/4/2007 17:21'!
clearSelection
	selection _ 0. ! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 5/4/2007 23:54'!
createNext
	| nextMetaNode created |
	nextMetaNode := self nextMetaNode.
	created _ OBColumn inPanel: panel metaNode: nextMetaNode node: self selectedNode.
	panel pushColumn: created.
	^created! !

!OBColumn methodsFor: 'printing' stamp: 'dr 4/5/2007 11:01'!
descriptor
	self isEmpty ifTrue: [^ 'empty'].
	"self hasSelection ifTrue: [^ self selectedNode name]."
	^ ''! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 5/6/2007 23:15'!
getChildren
	^ children _ self parent childNodes.
	! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 23:07'!
hasSelection
	^ self selection > 0! !

!OBColumn methodsFor: 'testing' stamp: 'PDC 6/26/2007 21:08'!
ifChildrenChanged: aBlock
	| oldChildren |
	oldChildren := children.
	self getChildren.
	(children = oldChildren) ifFalse: [aBlock value]! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 12/20/2004 22:09'!
includesNode: aNode
	^ children includes: aNode! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/28/2006 08:27'!
isEmpty
	^ children isEmpty! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 10/9/2006 15:41'!
isSelected: aNode
	^ self selectedNode == aNode! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 3/13/2007 00:52'!
keystroke: aChar from: aMorph
	| scan |
	self isEmpty ifTrue: [^ self].	
	scan _ self announcer announce: OBNodeCommandScan.
	scan processKeystroke: aChar withNode: self parent for: self.
	self hasSelection ifTrue: [scan 
								processKeystroke: aChar 
								withNode: self selectedNode 
								for: self].! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 5/20/2007 20:07'!
list
	^ children 
		ifNil: [#()]
		ifNotNil: [children collect: [:ea | parent displayStringForChild: ea]]! !

!OBColumn methodsFor: 'callbacks' stamp: 'dr 7/12/2007 17:19'!
listAt: index
	^parent displayStringForChild: (children at: index ifAbsent: [^ ''])! !

!OBColumn methodsFor: 'updating' stamp: 'dr 5/29/2007 10:30'!
listChanged 
	self refreshAndSignal: true! !

!OBColumn methodsFor: 'morphic' stamp: 'dr 5/29/2007 15:01'!
listMorph
	^ (OBPluggableListMorph
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			getListElementSelector: #listAt:;
			getListSizeSelector: #listSize;
			dragEnabled: self dragEnabled;
			dropEnabled: self dropEnabled;
			borderWidth: 0;
			autoDeselect: false;
			adoptPaneColor: panel defaultBackgroundColor;
			yourself! !

!OBColumn methodsFor: 'morphic' stamp: 'dr 4/24/2007 15:20'!
listSize
	^ children
		ifNil: [0]
		ifNotNil: [children size]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 3/13/2007 00:51'!
menu: aMenu 
	self isEmpty ifFalse: [self addCommandsToMenu: aMenu].
	^aMenu! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/6/2007 23:19'!
metaNode
	^ self parent metaNode! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 20:50'!
next
	^ self basicNext
		ifNil: [(self hasSelection and: [self shouldBeLast not])
				ifTrue: [self createNext]]
	! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:11'!
nextMetaNode
	^ self selectedNode metaNode! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:19'!
nodeChanged: ann
	(children includes: ann node) ifTrue: [self changed: #list]! !

!OBColumn methodsFor: 'updating' stamp: 'PDC 6/26/2007 21:11'!
nodeDeleted: announcement 
	"This gets called if an action causes the currently selected node to be
	deleted. "
	self selectedNode = announcement node
		ifFalse: [^ self].
	self 
		getChildren; 
		changed: #list; 
		selection: 0! !

!OBColumn methodsFor: 'callbacks' stamp: 'dr 5/18/2007 17:58'!
okToChange
	^(self announcer announce: OBAboutToChange) isVetoed not! !

!OBColumn methodsFor: 'accessing' stamp: 'dr 8/3/2006 17:07'!
panel

	^panel ! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/2/2007 23:50'!
parent
	^ parent 
		! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/18/2007 20:35'!
parent: aNode 
	self basicParent: aNode.
	self clearSelection.
	self changed: #list.
	panel clearAfter: self! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/18/2007 21:47'!
parent: pNode child: cNode 
	self basicParent: pNode.
	parent noteChild: cNode.
	switch refresh.
	self selectSilently: cNode.
	self changed: #list.
! !

!OBColumn methodsFor: 'selecting' stamp: 'dr 5/29/2007 14:54'!
preselectNodes! !

!OBColumn methodsFor: 'printing' stamp: 'cwp 2/11/2004 23:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self descriptor.
	aStream nextPut: $)! !

!OBColumn methodsFor: 'updating' stamp: 'dr 5/29/2007 10:29'!
refresh
	self refreshAndSignal: false! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:08'!
refresh: ann
	self refreshAndSignal: false! !

!OBColumn methodsFor: 'updating' stamp: 'PDC 6/26/2007 21:04'!
refreshAndSignal: aBoolean 
	| shouldSignal |
	shouldSignal := aBoolean.
	self isEmpty ifTrue: [^ self].
	self ifChildrenChanged:
			[self selectSilently: self selectedNode.
			self hasSelection ifFalse: [shouldSignal := true]].
	shouldSignal ifTrue: [self signalSelectionChanged].
	self changed: #list! !

!OBColumn methodsFor: 'selecting' stamp: 'dr 5/21/2007 11:14'!
select: aNode
	self selection: (children indexOf: (children detect: [:ea | ea = aNode] ifNone: []))! !

!OBColumn methodsFor: 'selecting' stamp: 'dr 7/4/2007 17:21'!
selectSilently: aNode
	selection _ children indexOf: aNode.
	selection = 0 ifTrue: [selection _ children indexOf: (children 
										detect: [:ea | ea name = aNode name] 
										ifNone: [nil])]! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 2/2/2004 21:54'!
selectedNode
	^ children at: self selection ifAbsent: [].! !

!OBColumn methodsFor: 'callbacks' stamp: 'dr 7/4/2007 17:21'!
selection
	^selection ifNil: [0]! !

!OBColumn methodsFor: 'callbacks' stamp: 'dr 7/4/2007 17:21'!
selection: anInteger
	(anInteger = 0 and: [ self basicNext notNil and: [ self basicNext hasSelection ] ])
		ifFalse: [ selection := anInteger ].
	self signalSelectionChanged.
	self changed: #selection ! !

!OBColumn methodsFor: 'updating' stamp: 'dr 5/29/2007 10:51'!
selectionChanged: ann
	! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 5/17/2007 23:45'!
setPanel: aPanel 
	panel _ aPanel.
	switch _ OBSwitch inColumn: self.
	self subscribe.
	children := #().
	self clearSelection! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 5/18/2007 20:40'!
setPanel: aPanel metaNode: aMetanode node: aNode 
	switch _ OBSwitch inColumn: self.
	panel _ aPanel.
	aNode metaNode: aMetanode.
	self basicParent: aNode.
	self clearSelection.
	self subscribe.
! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 16:27'!
shouldBeLast
	^ self hasSelection not or: [self nextMetaNode hasChildren not]! !

!OBColumn methodsFor: 'updating' stamp: 'dr 6/5/2007 17:34'!
signalSelectionChanged
	"TimeProfileBrowser spyOn: ["self announcer announce: (OBSelectionChanged column: self).
	self next ifNotNil: [self next preselectNodes].! !

!OBColumn methodsFor: 'updating' stamp: 'dr 5/29/2007 10:53'!
subscribe
	self announcer 
		observe: OBRefreshRequired send: #refresh: to: self;
		observe: OBNodeChanged send: #nodeChanged: to: self;
		observe: OBNodeDeleted send: #nodeDeleted: to: self;
		observe: OBChildrenChanged send: #childrenChanged: to: self.! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/17/2007 23:25'!
switch
	^ switch! !

!OBColumn methodsFor: 'accessing' stamp: 'dr 7/4/2007 16:51'!
switch: aSwitch
	switch := aSwitch! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 5/18/2007 20:24'!
switchFilter: aFilter
	"We should only be sending #changed: if the filter is actually different
	from the one the switch already has, but that causes some visually odd
	behavior because of the way OBRadioButtonBar resizes its self when updating,
	so for now we always just recreate the button bar."
	
	switch filter: aFilter.
	self changed: #switch! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 5/17/2007 23:24'!
wantsButton
	^ switch isActive! !

OBColumn subclass: #OBMultipleSelectionColumn
	instanceVariableNames: 'multipleSelection oldSelection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMultipleSelectionColumn methodsFor: 'building' stamp: 'dr 7/4/2007 15:51'!
buildOn: aBuilder
	^ aBuilder
		multipleSelectionColumn: self 
		with: [switch buildOn: aBuilder]! !

!OBMultipleSelectionColumn methodsFor: 'selecting' stamp: 'dr 5/29/2007 14:56'!
clearSelection
	super clearSelection.
	multipleSelection := Dictionary new.! !

!OBMultipleSelectionColumn methodsFor: 'callbacks' stamp: 'dr 6/12/2007 17:22'!
listAt: index
	^children at: index ifAbsent: [^ '']! !

!OBMultipleSelectionColumn methodsFor: 'morphic' stamp: 'dr 7/4/2007 15:50'!
listMorph
	^ (OBPluggableListMorphOfMany
		on: self
		list: #list
		primarySelection: #selection
		changePrimarySelection: #selection:
		listSelection: #listSelectionAt:
		changeListSelection: #listSelectionAt:put:		
		menu: #menu: 
		keystroke: #keystroke:from:)
			getListElementSelector: #listAt:;
			getListSizeSelector: #listSize;
			dragEnabled: self dragEnabled;
			dropEnabled: self dropEnabled;
			borderWidth: 0;
			autoDeselect: false;
			adoptPaneColor: panel defaultBackgroundColor;
			yourself! !

!OBMultipleSelectionColumn methodsFor: 'callbacks' stamp: 'dr 5/29/2007 14:55'!
listSelectionAt: anIndex
	^multipleSelection at: anIndex ifAbsent: [^false]! !

!OBMultipleSelectionColumn methodsFor: 'callbacks' stamp: 'dr 7/4/2007 17:21'!
listSelectionAt: anIndex put: aValue 
     | isSelected |
     isSelected := multipleSelection at: anIndex ifAbsent: [false].

     "if we did not de-select the primary selection"
     (aValue not and: [anIndex = selection]) ifFalse: [
	     multipleSelection at: anIndex put: aValue.
          "if we de-select this item"
	     (isSelected = true and: [aValue not]) ifTrue: [
		    self announcer announce: ((OBSelectionChanged column: self) deselectedNode: (children at: anIndex)).
	     ] ifFalse: [ "if we select an new item => check all old items"
	         (multipleSelection associationsDo: [:assoc | (assoc value = true and: [self paneExistsForSelection: assoc key]) ifFalse: [
			    multipleSelection at: assoc key put: false.
			]])
	     ]
     ].! !

!OBMultipleSelectionColumn methodsFor: 'testing' stamp: 'dr 7/4/2007 15:50'!
paneExistsForSelection: anIndex
    ^panel definitionPanel includesNode: (self getChildren at: anIndex).! !

!OBMultipleSelectionColumn methodsFor: 'selecting' stamp: 'dr 7/4/2007 15:50'!
preselectNodes
     self getChildren keysAndValuesDo: [:idx :node | (panel definitionPanel includesNode: node) ifTrue: [multipleSelection at: idx put: true]]. ! !

!OBMultipleSelectionColumn methodsFor: 'callbacks' stamp: 'dr 7/4/2007 15:50'!
selection: anInteger
     "de-selected an item"
	anInteger = 0 ifTrue: [
         "primarySelection := oldPrimarySelection. self halt."
         "selection keysAndValuesDo: [:index :value | value ifTrue: [primarySelection := index]]."
	] ifFalse: [ 
	    oldSelection := selection.
	    selection _ anInteger.
	    (multipleSelection isEmpty and: [self = panel columns last]) ifTrue: [multipleSelection at: anInteger put: true].
         self signalSelectionChanged.
	]. 
	self changed: #selection.
! !

OBColumn subclass: #OBSmartGroupColumn
	instanceVariableNames: 'smartGroup'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBSmartGroupColumn methodsFor: 'updating' stamp: 'dr 6/7/2007 16:30'!
childrenChanged: announcement
	(announcement node isKindOf: OBSmartGroupNode) ifTrue:
		[self refresh].! !

!OBSmartGroupColumn methodsFor: 'nodes' stamp: 'dr 7/4/2007 15:50'!
nextMetaNode
    ^panel root metaNode! !

!OBSmartGroupColumn methodsFor: 'accessing' stamp: 'dr 4/5/2007 14:48'!
parent
	^smartGroup! !

!OBSmartGroupColumn methodsFor: 'accessing' stamp: 'dr 4/16/2007 16:16'!
selectedNode
	^children at: self selection ifAbsent: [panel root].! !

!OBSmartGroupColumn methodsFor: 'nodes' stamp: 'dr 7/4/2007 15:50'!
selectionChanged: ann
    (ann node respondsTo: #alias) ifTrue: [panel jumpTo: ann node alias].! !

!OBSmartGroupColumn methodsFor: 'initializing' stamp: 'dr 4/5/2007 14:48'!
setPanel: aPanel metaNode: aMetaNode node: aNode
    super setPanel: aPanel metaNode: aMetaNode node: aNode.
    smartGroup := aNode
    ! !

!OBSmartGroupColumn methodsFor: 'updating' stamp: 'dr 5/29/2007 10:58'!
subscribe
    super subscribe.
    self announcer observe: OBSelectionChanged send: #selectionChanged: to: self.! !

Object subclass: #OBCommand
	instanceVariableNames: 'target requestor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Commands'!

!OBCommand class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 23:29'!
on: target for: requestor
	^ self new setTarget: target requestor: requestor! !

!OBCommand class methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:58'!
takesNodes
	^ true! !

!OBCommand class methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:58'!
takesText
	^ false! !

!OBCommand methodsFor: 'user interface' stamp: 'cwp 6/8/2007 20:45'!
addItemToMenu: aMenu
	self isActive ifTrue:
		[aMenu
			add: self labelWithKeystroke
			target: self 
			selector: #execute
			enabled: self isEnabled
			icon: self icon]! !

!OBCommand methodsFor: 'accessing' stamp: 'lr 4/25/2007 09:27'!
buttonLabel
	^ self label! !

!OBCommand methodsFor: 'execution' stamp: 'cwp 10/15/2006 14:12'!
execute
	! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:44'!
group
	^ #general! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 18:45'!
icon
	^ nil! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 22:05'!
isActive
	^ false! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 19:31'!
isEnabled
	^ true! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
keystroke
	^ nil
! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
label
	self subclassResponsibility
! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:19'!
labelWithKeystroke
	^ self keystroke
		ifNil: [self label]
		ifNotNil: [self label, ' (', self keystroke asString, ')']! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/14/2006 23:39'!
on: anObject for: aRequestor
	^ self class on: anObject for: aRequestor! !

!OBCommand methodsFor: 'user interface' stamp: 'cwp 10/15/2006 12:37'!
perform: aSelector orSendTo: anObject
	self perform: aSelector! !

!OBCommand methodsFor: 'user interface' stamp: 'cwp 10/9/2006 13:49'!
select: aNode with: anAnnouncer
	(anAnnouncer announce: OBAboutToChange)
		isVetoed ifFalse: [anAnnouncer announce: (OBSelectingNode node: aNode)]! !

!OBCommand methodsFor: 'initialize-release' stamp: 'cwp 10/14/2006 23:31'!
setTarget: anObject requestor: aRequestor
	target _ anObject.
	requestor _ aRequestor! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/14/2006 19:38'!
takesNodes
	^ true! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/14/2006 19:37'!
takesText
	^ false! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 9/27/2006 22:53'!
wantsButton
	"Put this here for compatibility. Eventually this will be obsolete"

	^ false
! !

!OBCommand methodsFor: 'testing' stamp: 'dr 6/28/2007 16:24'!
wantsTabButton
    ^false! !

OBCommand subclass: #OBPluggableCommand
	instanceVariableNames: 'action active enabled label keystroke icon buttonLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Commands'!

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 6/18/2006 18:53'!
action: aValuable
	^ self new action: aValuable! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 13:50'!
action: aValuable active: aValuable2
	^ self new 
		action: aValuable;
		active: aValuable2;
		yourself! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 6/24/2006 15:43'!
new
	^ self basicNew initialize! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 00:57'!
takesNodes
	^ true! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 00:58'!
takesText
	^ false! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'dc 4/28/2007 11:36'!
action: aValuable
	action := aValuable! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'dc 4/28/2007 11:36'!
active: aValuable
	active := aValuable! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:41'!
buttonLabel
	^ buttonLabel! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:40'!
buttonLabel: aString
	buttonLabel _ aString! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:31'!
enabled: aBlock
	enabled _ aBlock! !

!OBPluggableCommand methodsFor: 'execute' stamp: 'dkh 5/31/2007 13:53'!
execute
	^ action valueWithPossibleArgs: (Array with: target with: requestor)! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:46'!
group
	^ #general! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:37'!
icon
	^ icon! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:37'!
icon: anIcon
	icon _ anIcon! !

!OBPluggableCommand methodsFor: 'initialize-release' stamp: 'cwp 11/2/2006 00:39'!
initialize
	action _ [].
	active _ [true].
	enabled _ [true].
	label _ 'a command'.! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'dkh 5/31/2007 13:53'!
isActive
	^ active valueWithPossibleArgs: (Array with: target with: requestor)! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'dkh 5/31/2007 13:53'!
isEnabled
	^ enabled valueWithPossibleArgs: (Array with: target with: requestor)! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 19:04'!
keystroke
	^ keystroke! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 19:04'!
keystroke: aCharacter
	keystroke _ aCharacter! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 19:04'!
label
	^ label! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 18:58'!
label: aString
	label _ aString! !

!OBPluggableCommand methodsFor: 'morphic' stamp: 'cwp 6/24/2006 19:05'!
labelWithKeystroke
	^keystroke
		ifNil: [label]
		ifNotNil: [label, ' (', keystroke asString, ')']! !

!OBPluggableCommand methodsFor: 'converting' stamp: 'cwp 10/15/2006 12:46'!
on: aNode for: aRequestor 
	| inst |
	inst _ self copy.
	inst setTarget: aNode requestor: aRequestor.
	^ inst! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:43'!
takesNodes
	^ true! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 6/25/2006 00:43'!
useLineAfter
	^ false! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 6/24/2006 19:46'!
wantsButton
	^ self buttonLabel notNil! !

Object subclass: #OBDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBDefinition commentStamp: 'cwp 1/7/2005 23:38' prior: 0!
The responsibility of a definition is to express a node's composition textually and respond to changes in the text by updating the node it represents. For example,a file browser might use a file definition to allow editing of a files contents.!

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:51'!
accept: aText notifying: aController
	^ self text: aText! !

!OBDefinition methodsFor: 'updating' stamp: 'cwp 7/25/2006 18:26'!
asAnnouncement
	^ OBDefinitionChanged definition: self! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:08'!
doItContext
	^ nil! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 2/28/2005 23:23'!
doItReceiver
	| class |
	class _ self selectedClass.
	^ class
		ifNotNil: [class theNonMetaClass]
		ifNil: [FakeClassPool new]! !

!OBDefinition methodsFor: 'confirmation' stamp: 'PDC 6/26/2007 22:54'!
selectedClass
	^nil! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
selection
	^ 1 to: 0! !

!OBDefinition methodsFor: 'updating' stamp: 'cwp 6/4/2006 18:53'!
signalChange
	OBAnnouncer current announce: (OBDefinitionChanged definition: self)! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text
	^ ''! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text: aText
	^ false! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
textSelection
	^ self selection! !

Object subclass: #OBFilter
	instanceVariableNames: 'metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBFilter commentStamp: 'cwp 3/4/2004 21:53' prior: 0!
A browser's metagraph defines the way in which the user may traverse the graph of objects which make up the browser's domain. But it's not always desirable to have all the nodes made available by the metagraph to be visible in the browser. An filter provides a strategy for filtering out some of the nodes from the display. 

OBFilter provides a "null" filter - one that does no filtering at all - and serves as a superclass for other filters. !

!OBFilter methodsFor: 'user interface' stamp: 'cwp 10/24/2006 10:18'!
buildOn: aBuilder
	! !

!OBFilter methodsFor: 'filtering' stamp: 'cwp 5/20/2007 01:56'!
displayString: aString forNode: aNode
	^ aString! !

!OBFilter methodsFor: 'filtering' stamp: 'cwp 5/20/2007 20:05'!
displayString: aString forParent: pNode child: cNode
	^ aString! !

!OBFilter methodsFor: 'filtering' stamp: 'cwp 5/6/2007 01:53'!
edgesFrom: aCollection forNode: aNode
	^ aCollection! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:48'!
metaNode
	^ metaNode! !

!OBFilter methodsFor: 'filtering' stamp: 'cwp 5/6/2007 01:53'!
nodesFrom: aCollection forNode: aNode
	^ aCollection! !

!OBFilter methodsFor: 'filtering' stamp: 'cwp 5/8/2007 00:10'!
noteParent: pNode child: cNode! !

!OBFilter methodsFor: 'initalizing' stamp: 'cwp 2/9/2004 20:54'!
setMetaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:06'!
wantsButton
	^ false! !

!OBFilter methodsFor: 'testing' stamp: 'cwp 5/17/2007 23:07'!
wantsSwitch
	^ false! !

OBFilter subclass: #OBModalFilter
	instanceVariableNames: 'selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBModalFilter commentStamp: 'cwp 1/7/2005 23:39' prior: 0!
OBModalFilter is used to implement the 'instance/?/class' buttons in a standard class browser. In functional terms it filters the nodes of a column according to the edge of the metagraph that they correspond to. OBModalFilter displays an OBRadioButtonBar in its column's pane, with one button per edge. Only nodes from the currently selected edge are allowed in the column.

iVars:

selection	- the currently selected edge!

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 5/17/2007 22:57'!
activate
	! !

!OBModalFilter methodsFor: 'as yet unclassified' stamp: 'cwp 5/10/2007 23:41'!
edgesFrom: aCollection forNode: aNode
	^ Array with: (aCollection at: self selection)! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:28'!
list
	^ metaNode edges collect: [:edge | edge label]! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 5/8/2007 00:16'!
noteParent: pNode child: cNode
	pNode metaNode edges withIndexDo:
		[:edge :index | 
		cNode metaNode = edge metaNode
			ifTrue: [selection _ index.
					self changed: #selection]]! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/9/2004 21:10'!
selection
	^ selection ifNil: [selection _ 1]! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 5/17/2007 23:43'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection.
	! !

!OBModalFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:08'!
wantsButton
	^ true! !

!OBModalFilter methodsFor: 'testing' stamp: 'cwp 5/17/2007 23:07'!
wantsSwitch
	^ true! !

Object subclass: #OBMetaEdge
	instanceVariableNames: 'label metaNode selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaEdge commentStamp: 'cwp 1/7/2005 23:20' prior: 0!
An OBMetaEdge is an edge in the browser's metagraph. It represents a message sent to a node to obtain further nodes. It is refered to by the "parent" metanode, and refers to the "child" metanode.

iVars:

label 		-  a string describing the metaNode, for filters which allow the user 
			   to choose which edges to follow
selector	- when a node is selected by the user, this message will be 
			  sent to it to obtain its children
metaNode 	- a MetaNode corresponding to the nodes answered by the above message
		!

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:35'!
label: aString selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 5/4/2007 22:53'!
selector: aSymbol
	^ self selector: aSymbol metaNode: nil! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 22:32'!
selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aSelector asString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 5/4/2007 22:56'!
-> aNode
	metaNode _ aNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 5/4/2007 23:10'!
/ aString
	label _ aString! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
label
	^ label! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
metaNode
	^ metaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 3/3/2004 00:12'!
nodesForParent: aNode
	^ (aNode perform: selector) do: [:ea | ea metaNode: metaNode]
! !

!OBMetaEdge methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:03'!
printOn: aStream
	aStream nextPutAll: selector printString, '->'. 
	metaNode shortPrintOn: aStream.! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
selector
	^ selector! !

!OBMetaEdge methodsFor: 'initializing' stamp: 'cwp 2/7/2004 22:36'!
setLabel: aString selector: aSelector metaNode: aMetaNode
	label _ aString.
	selector _ aSelector.
	metaNode _ aMetaNode! !

Object subclass: #OBMetaNode
	instanceVariableNames: 'name filters edges ancestrySelector displaySelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaNode commentStamp: 'cwp 1/7/2005 23:23' prior: 0!
A MetaNode represents a hypothetical node in the browser's domain graph.

iVars:

filterClass			- the class used to filter this hypothetical node's children
columnClass		- the class of column used to display this node's children
edges				- messages that could be sent to the node to obtain children
actors				- providers of functionality for manipulating the node
displaySelector		- the message used to retreive a nodes display name!

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/6/2004 21:06'!
named: aString
	^ self new name: aString! !

!OBMetaNode methodsFor: 'children' stamp: 'PDC 6/26/2007 21:18'!
- aSelector
	^ edges add: (OBMetaEdge selector: aSelector).
	! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 5/18/2007 00:20'!
addFilter: aFilter
	aFilter setMetaNode: self.
	filters add: aFilter! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 5/7/2007 23:01'!
allChildrenForNode: aNode
	^ edges gather: [:edge | edge nodesForParent: aNode]! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 5/11/2007 00:40'!
ancestrySelector
	^ ancestrySelector ifNil: [#isDescendantOf:]! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 5/11/2007 00:27'!
ancestrySelector: anObject
	ancestrySelector _ anObject! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/9/2004 20:28'!
childAt: aSelector labeled: aString put: aMetaNode
	edges add: (OBMetaEdge label: aString selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:35'!
childAt: aSelector put: aMetaNode
	edges add: (OBMetaEdge selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 5/11/2007 01:05'!
childOf: parent ancestorOf: descendant
	^ (self allChildrenForNode: parent)
		detect: [:ea | ea isAncestorOf: descendant using: ea metaNode ancestrySelector]
		ifNone: [nil]! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:44'!
children
	^ edges collect: [:edge | edge metaNode]! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 5/6/2007 23:46'!
childrenForNode: aNode
	| chosenEdges |
	chosenEdges _ self edgesForNode: aNode.
	^ chosenEdges gather: 
		[:edge | 
		filters 
			inject: (edge nodesForParent: aNode)
			into: [:list :filter | filter nodesFrom: list forNode: aNode]].! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 21:41'!
displaySelector
	^ displaySelector ifNil: [displaySelector _ #name]! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 21:42'!
displaySelector: aSelector
	displaySelector _ aSelector! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 5/20/2007 01:51'!
displayStringForNode: aNode
	^ filters
		inject: (aNode perform: self displaySelector)
		into: [:string :filter | filter displayString: string forNode: aNode]! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 5/20/2007 20:05'!
displayStringForParent: pNode child: cNode
	^ filters
		inject: cNode displayString
		into: [:string :filter | filter displayString: string forParent: pNode child: cNode]! !

!OBMetaNode methodsFor: 'metagraph' stamp: 'cwp 2/9/2004 21:12'!
edges
	^ edges! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 5/6/2007 23:47'!
edgesForNode: aNode 
	^ filters 
		inject: edges 
		into: [:list :filter | filter edgesFrom: list forNode: aNode].! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:38'!
hasChildren
	^ edges isEmpty not! !

!OBMetaNode methodsFor: 'initializing' stamp: 'cwp 5/6/2007 00:30'!
initialize
	filters _ OrderedCollection new.
	edges _ OrderedCollection new.
! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:00'!
metaNode
	^ self! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:54'!
name
	^ name ifNil: ['MetaNode']! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:05'!
name: aString
	name _ aString! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:22'!
nodesForParent: aNode
	^ edges gather: [:edge | edge nodesForParent: aNode]! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 5/8/2007 00:14'!
noteParent: pNode child: cNode
	filters do: [:ea | ea noteParent: pNode child: cNode]! !

!OBMetaNode methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:01'!
printOn: aStream
	aStream nextPutAll: self name;cr.
	edges do: [:e | e printOn: aStream. aStream cr].! !

!OBMetaNode methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:01'!
shortPrintOn: aStream
	aStream nextPutAll: self name.! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 5/18/2007 20:23'!
switchFilter
	^ filters 
		detect: [:ea | ea wantsSwitch]
		ifNone: [nil]! !

Object subclass: #OBNode
	instanceVariableNames: 'metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBNode commentStamp: 'cwp 3/4/2004 22:20' prior: 0!
A node is a wrapper for an object in the browser's domain graph. OBNode is an abstract superclass for concrete nodes which might appear in the browser.

iVars:

metaNode - the MetaNode which produced this node!

OBNode subclass: #OBCollectionNode
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBCollectionNode commentStamp: 'cwp 1/7/2005 23:31' prior: 0!
OBCollectionNode is a trivial wrapper for a collection of nodes. It is typically used as an artificial root node for metagraphs that have no natural root.!

!OBCollectionNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/15/2004 23:17'!
on: aCollection
	^ self new setCollection: aCollection! !

!OBCollectionNode methodsFor: 'navigating' stamp: 'cwp 4/15/2007 23:20'!
addChild: aNode
	(collection includes: aNode)
		ifFalse: [collection _ collection copyWith: aNode]! !

!OBCollectionNode methodsFor: 'navigating' stamp: 'cwp 3/15/2004 23:19'!
children
	^ collection! !

!OBCollectionNode methodsFor: 'ancestry' stamp: 'cwp 4/3/2007 22:34'!
isAncestorOf: aNode
	^ collection anySatisfy: [:ea | ea = aNode or: [ea isAncestorOf: aNode]]! !

!OBCollectionNode methodsFor: 'displaying' stamp: 'cwp 3/15/2004 23:19'!
name
	^ collection species name! !

!OBCollectionNode methodsFor: 'initalizing' stamp: 'cwp 3/15/2004 23:18'!
setCollection: aCollection
	collection _ aCollection! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 22:11'!
fromAssociation: anAssociationNode
	^ self on: anAssociationNode value! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'PDC 6/26/2007 21:39'!
on: anObject
	self subclassResponsibility ! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:57'!
= other
	^ self class = other class and: [self name = other name]! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 13:49'!
accept: aText notifying: aController
	^ self text: aText! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
ancestrySelector
	^ #isDescendantOf: ! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:18'!
announce: anObject
	^ OBAnnouncer current ifNotNilDo: [:announcer | announcer announce: anObject]! !

!OBNode methodsFor: 'updating' stamp: 'cwp 3/13/2007 00:18'!
announceChangedWith: anObject
	anObject announce: (OBNodeChanged node: self)! !

!OBNode methodsFor: 'updating' stamp: 'cwp 3/13/2007 00:21'!
announceDeletionWith: anObject
	anObject announce: (OBNodeDeleted node: self)
! !

!OBNode methodsFor: 'updating' stamp: 'cwp 3/13/2007 00:34'!
announceSelectionWith: anObject
	(anObject announce: OBAboutToChange)
		isVetoed ifFalse: [anObject announce: (OBSelectingNode node: self)]! !

!OBNode methodsFor: 'displaying' stamp: 'dr 3/8/2007 11:02'!
asString
	^self displayString! !

!OBNode methodsFor: 'public' stamp: 'cwp 5/11/2007 00:03'!
childAncestorOf: aNode
	^ self metaNode childOf: self ancestorOf: aNode! !

!OBNode methodsFor: 'public' stamp: 'cwp 5/6/2007 01:49'!
childNodes
	^ metaNode childrenForNode: self! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 7/14/2006 12:37'!
correspondsWith: aNode
	^ self displayString = aNode displayString! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/22/2004 20:48'!
definition
	^ self! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 01:06'!
demandSelection
	^ OBAnnouncer current announce: (OBNodeCreated node: self)! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 5/20/2007 01:57'!
displayString
	^ metaNode displayStringForNode: self! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 5/20/2007 20:02'!
displayStringForChild: aNode
	^ metaNode displayStringForParent: self child: aNode! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:57'!
hash
	^ self name hash! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 5/11/2007 00:18'!
isAncestorOf: aNode
	^ self isAncestorOf: aNode using: self ancestrySelector! !

!OBNode methodsFor: 'ancestry' stamp: 'dr 7/16/2007 11:32'!
isAncestorOf: aNode using: aSelector
	^ aNode perform: aSelector with: self! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
isDescendantOf: aNode
	^ false! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode
	^ metaNode! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/10/2003 22:27'!
name
	self subclassResponsibility! !

!OBNode methodsFor: 'public' stamp: 'cwp 5/18/2007 20:33'!
noteChild: aNode
	metaNode noteParent: self child: aNode! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/7/2004 21:29'!
referenceForMethod: selector ofClass: className
	| classIsMeta symbol |
	classIsMeta _ className endsWith: ' class'.
	symbol _ classIsMeta ifTrue: [(className allButLast: 6) asSymbol] ifFalse: [className].
	^ MethodReference new 
		setClassSymbol: symbol
		classIsMeta: classIsMeta
		methodSymbol: selector
		stringVersion: symbol, '>>', selector! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:15'!
signalChanged
	self announce: (OBNodeChanged node: self)! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:15'!
signalChildrenChanged
	self announce: (OBChildrenChanged node: self)! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:15'!
signalDeletion
	self announce: (OBNodeDeleted node: self)
! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 13:59'!
signalSelection
	(self announce: OBAboutToChange)
		isVetoed ifFalse: [self announce: (OBSelectingNode node: self)]! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:27'!
text
	^ ''! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/14/2004 14:09'!
text: aText
	^ false! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 18:30'!
textSelection
	^ 1 to: 0! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 8/28/2004 00:14'!
title
	^ nil! !

Object subclass: #OBPanel
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBPanel commentStamp: 'cwp 1/7/2005 23:23' prior: 0!
A panel is an object that manages part of the browser's window. It provides a protocol for receiving notifications when the current domain node changes, and reacts to these changes.!

OBPanel subclass: #OBCodeTabPanel
	instanceVariableNames: 'definitionPanel selection commands morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBCodeTabPanel methodsFor: 'configuration' stamp: 'dr 7/12/2007 14:27'!
cellInset
    ^5! !

!OBCodeTabPanel methodsFor: 'callbacks' stamp: 'dr 6/22/2007 17:00'!
color
	^browser defaultBackgroundColor ! !

!OBCodeTabPanel methodsFor: 'callbacks' stamp: 'dr 7/3/2007 15:04'!
commands
	| tmpCommands |
	self node ifNil: [^ #()].
	tmpCommands _ (self announce: OBNodeCommandScan) commandsOn: self node for: self. 
	commands := (tmpCommands select: [:ea | ea wantsTabButton]) asSortedCollection: [:a :b | (a priority < b priority) or: [a priority = b priority and: [a label <= b label]]].
	^commands! !

!OBCodeTabPanel methodsFor: 'morphic' stamp: 'dr 7/12/2007 14:27'!
createTabPanel
	| tabPanel |
	tabPanel _ OBRadioButtonBar
					on: self
					list: #list
					selected: #selection
					changeSelected: #selection:.
	tabPanel height: 23;
		      hResizing: #shrinkWrap;
		      vResizing: #shrinkWrap;
		      cellInset: self cellInset;	
	           cellPositioning: #topLeft.	
	tabPanel adoptPaneColor: browser defaultBackgroundColor darker darker darker.				
	^tabPanel! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 6/28/2007 10:58'!
definitionPanel
    ^definitionPanel! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 6/28/2007 10:45'!
definitionPanel: aDefinitionPanel
    definitionPanel := aDefinitionPanel! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 7/12/2007 14:55'!
executeCommand
     (commands at: self selection) execute.! !

!OBCodeTabPanel methodsFor: 'initializing' stamp: 'dr 6/29/2007 16:26'!
initialize
    super initialize.
    selection := 1.! !

!OBCodeTabPanel methodsFor: 'testing' stamp: 'dr 6/28/2007 10:37'!
isTab
    ^true! !

!OBCodeTabPanel methodsFor: 'callbacks' stamp: 'dr 7/12/2007 14:42'!
list 
    | labelSelector |
    labelSelector := definitionPanel morph bounds width < self maxWidth ifTrue: [#shortLabel] ifFalse: [#label].
    ^self commands collect: [:ea | ea perform: labelSelector]! !

!OBCodeTabPanel methodsFor: 'morphic' stamp: 'dr 7/12/2007 14:32'!
maxWidth
    ^self commands inject: 0 into: [:width :cmd | width := width + (cmd label size * 7 + self cellInset). width]! !

!OBCodeTabPanel methodsFor: 'morphic' stamp: 'dr 7/12/2007 14:52'!
morph
    morph ifNil: [morph := self createTabPanel].
    ^morph! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 6/28/2007 16:17'!
node
    ^self definitionPanel node! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 6/22/2007 17:19'!
panel
    ^self! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 6/28/2007 16:07'!
selection
	^selection! !

!OBCodeTabPanel methodsFor: 'accessing' stamp: 'dr 7/12/2007 14:56'!
selection: anInteger
	selection := anInteger.
     self executeCommand.
	self changed: #selection.! !

!OBCodeTabPanel methodsFor: 'callbacks' stamp: 'dr 6/22/2007 17:01'!
selectionChanged: ann
	self changed: #commands! !

!OBCodeTabPanel methodsFor: 'callbacks' stamp: 'dr 6/29/2007 14:21'!
subscribe
	self announcer
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self! !

OBPanel subclass: #OBColumnPanel
	instanceVariableNames: 'root columns minPanes maxPanes smartGroup'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBColumnPanel commentStamp: 'cwp 12/6/2004 23:50' prior: 0!
OBColumnPanel handles navigation around the nodes of the object graph. It maintains a list of columns, which track the user's path through the node tree. As nodes are selected, additional columns are added to the list, which appear as panes on the right of the panel. 

iVars:

columns	- A collection of OBColumns, each of which manages a single pane in the scroller.
minPanes - The minimum number of panes that should ever be visible.
maxPanes - The maximum number of panes that should ever be visible.
!

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
minPanes: min maxPanes: max
	^ self basicNew setMinPanes: min maxPanes: max! !

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
new
	^ self minPanes: 1 maxPanes: 1! !

!OBColumnPanel methodsFor: 'building' stamp: 'cwp 3/16/2007 23:09'!
buildOn: aBuilder
	^ aBuilder 
		columnPanel: self 
		with: [columns do: [:ea | ea buildOn: aBuilder]]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 5/11/2007 01:08'!
clearAfter: aColumn 
	| start |
	aColumn ifNotNil:
		[start _ (columns indexOf: aColumn) + 1.
		start to: columns size do: [:i | (columns at: i) clear]]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 2/28/2006 18:12'!
columnAfter: aColumn 
	^ [self columns after: aColumn]
		on: Error
		do: [:err | nil]
! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 2/28/2006 19:07'!
columnBefore: aColumn 
	^ self columnBefore: aColumn ifAbsent: [nil]

! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 2/28/2006 19:07'!
columnBefore: aColumn ifAbsent: aBlock
	^ [self columns before: aColumn]
		on: Error
		do: [:err | aBlock value]


! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns
	^columns! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns: anObject
	columns := anObject! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 22:22'!
currentColumn
	^self columns reversed detect: [:ea | ea hasSelection] ifNone: []! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:22'!
currentNode
	^ self currentColumn ifNotNilDo: [:column | column selectedNode]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:23'!
currentOrRootNode
	^(self columns reversed detect: [:ea | ea hasSelection]
		ifNone: [^self columns first parent]) selectedNode! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/26/2004 21:55'!
defaultBackgroundColor
	^ browser defaultBackgroundColor! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 10:36'!
definitionPanel
    ^browser definitionPanel! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'dr 6/14/2007 10:19'!
emptyColumn
     | columnClass methodColumnIdx |
     self flag: #toRefactor.
     columnClass := OBColumn.
     methodColumnIdx := 3.
     (self columns first isKindOf: OBSmartGroupColumn) ifTrue: [
	    methodColumnIdx := 4.
	].

     self columns size = methodColumnIdx ifTrue: [columnClass := OBMultipleSelectionColumn].
	^columnClass inPanel: self! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 6/5/2006 23:10'!
hasSelection
	^ false! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 6/9/2006 23:18'!
hopTo: aNode 
	| column |
	column := self columns last.
	[column refreshAndSignal: false; includesNode: aNode]
		whileFalse: [column := self 
						columnBefore: column 
						ifAbsent: [^ self jumpTo: aNode]].
	column select: aNode! !

!OBColumnPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:10'!
isNavigation
	^ true! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 2/25/2007 20:40'!
isVariableHeight
	^ true! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'dr 7/16/2007 11:33'!
jumpTo: aNode 
	| column |
	self selectAncestorsOf: aNode.
	column _ self columns reversed 
				detect: [:ea | ea selectedNode = aNode] 
				ifNone: [^ self].
	self clearAfter: column next.
	self announcer announce: (OBSelectionChanged column: column)! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 11/16/2004 23:34'!
jumpToRoot
	^ self columns first selection: 0! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:51'!
labelString
	| label |
	self columns reversed do: 
		[:ea | 
		label := ea selectedNode ifNotNilDo: [:node | node title].
		label ifNotNil: [^ label]].
	^ nil
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:29'!
maxPanes
	^maxPanes! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:54'!
minPanes
	^ minPanes! !

!OBColumnPanel methodsFor: 'updating' stamp: 'dr 5/22/2007 16:24'!
nodeCreated: announcement
	^ self hopTo: announcement node! !

!OBColumnPanel methodsFor: 'updating' stamp: 'PDC 6/26/2007 22:45'!
nodeDeleted: ann 
	ann node = self root ifFalse: [^ self].
	self columns first clear.
	self announcer announce: (OBSelectionChanged column: self)! !

!OBColumnPanel methodsFor: 'updating' stamp: 'PDC 6/26/2007 22:47'!
okToReclaimPane
	^ columns size > minPanes 
		and: [columns last isEmpty or: [(columns at: columns size - 1) shouldBeLast]]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 6/4/2006 00:39'!
popColumn
	self announcer unsubscribe: self columns removeLast.
! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 3/25/2007 00:24'!
pushColumn: aColumn 
	self columns addLast: aColumn.
	self changed: #columns.
! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 5/28/2007 01:35'!
reclaimPanes
	| old |
	old := columns size.
	[self okToReclaimPane] whileTrue: [self popColumn].
	columns size = old ifFalse: [self changed: #columns]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 9/24/2005 22:47'!
root
	^ root! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'dr 7/16/2007 11:29'!
selectAncestorsOf: aNode 
	| path current |
	path := OrderedCollection new.
	current := root.
	[current isNil or: [current = aNode]] whileFalse: 
			[current := current childAncestorOf: aNode.
			path add: current].
	current ifNotNil: [self selectPath: path]! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 6/4/2006 01:03'!
selectNode: announcement
	^ self hopTo: announcement node! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'dr 6/12/2007 15:50'!
selectPath: aCollection
	| column parent |
	column_ smartGroup ifNil: [columns first] ifNotNil: [columns second].
	parent _ self root.
	aCollection do:
		[:node |
		column parent: parent child: node.
		column _ column next.
		parent _ node].! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 6/8/2006 01:35'!
selected: aColumn 
	aColumn next ifNotNilDo: 
		[:next | 
		next parent: aColumn selectedNode].! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 3/11/2007 21:08'!
selectedNode
	^ self currentNode! !

!OBColumnPanel methodsFor: 'updating' stamp: 'dr 6/12/2007 14:57'!
selectionChanged: ann
     (ann column isKindOf: OBSmartGroupColumn) ifTrue: [^self].

	ann column hasSelection
		ifTrue: [self selected: ann column]
		ifFalse: [self clearAfter: ann column].
	self reclaimPanes.	
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'dvf 9/5/2005 18:08'!
selectionPath
	^(self columns 
		collect: [:e | e parent] 
		thenSelect: [:e | e notNil]) allButFirst! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'dr 6/12/2007 15:37'!
setMetaNode: aMetaNode node: aNode 
	| metaNode |
	root := aNode.
	metaNode := aMetaNode.
	
	aMetaNode name = 'SmartGroup' ifTrue: [
	    metaNode := (aMetaNode children detect: [:ea | ea name = 'Environment']).	
	    smartGroup := OBSmartRootNode default.
	    smartGroup panel: self.
	    self pushColumn: (OBSmartGroupColumn 
				inPanel: self
				metaNode: aMetaNode
				node: smartGroup).		
	].
	root metaNode: metaNode.
	
	self pushColumn: (OBColumn inPanel: self metaNode: metaNode node: aNode).	
	minPanes - self columns size 
		timesRepeat: [self pushColumn: self emptyColumn]! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:54'!
setMinPanes: min maxPanes: max
	columns _ OrderedCollection new.
	minPanes _ min.
	maxPanes _ max! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/18/2004 00:00'!
sizing
	^ (columns size max: minPanes) min: maxPanes! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'dr 4/5/2007 14:27'!
smartGroup
    ^smartGroup! !

!OBColumnPanel methodsFor: 'updating' stamp: 'dr 5/22/2007 16:22'!
subscribe
	self announcer
		observe: OBSelectingNode send: #selectNode: to: self;
		observe: OBSelectionChanged send: #selectionChanged: to: self;
		observe: OBNodeCreated send: #nodeCreated: to: self;
		observe: OBNodeDeleted send: #nodeDeleted: to: self.! !

!OBColumnPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 20:36'!
vResizing
	^ #spaceFill! !

OBPanel subclass: #OBFixedButtonPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBFixedButtonPanel commentStamp: 'cwp 1/7/2005 23:35' prior: 0!
OBFixedButtonPanel displays a horizontal row of buttons. In contrast to OBVarButtonPanel, the buttons do not change as nodes are selected in the navigation panel; instead they are enabled and disabled according to whether the actions they represent are applicable to the selected node.!

!OBFixedButtonPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 22:13'!
buildOn: aBuilder
	^ aBuilder
		fixedButtonPanel: self
		with: []! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 20:12'!
color
	^ browser defaultBackgroundColor ! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:15'!
commands
	| commands node |
	node _ self currentNode.
	node ifNil: [^ #()].
	commands _ (self announce: OBNodeCommandScan) commandsOn: node for: self.
	^ (commands select: [:ea | ea wantsButton]) asSortedCollection: [:a :b | a label <= b label]! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:15'!
currentNode
	^ browser currentOrRootNode ! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:14'!
isSelected: aNode
	^ true! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 2/25/2007 20:40'!
isVariableHeight
	^ false! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:14'!
selectionChanged: ann
	self changed: #commands! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 6/3/2006 22:14'!
subscribe
	self announcer
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self! !

!OBFixedButtonPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 22:16'!
vResizing
	^ #rigid! !

OBPanel subclass: #OBMultipleDefinitionPanel
	instanceVariableNames: 'panels'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 5/21/2007 16:44'!
aboutToChange: ann
   " self panels first aboutToChange: ann."! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/28/2007 11:13'!
addPanel: aPanel
    "remove empty standard panel"
    (panels size = 1 and: [panels first node isNil]) ifTrue: [panels := OrderedCollection new].

    aPanel owner: self.
    panels add: aPanel.

    "make sure that the first panel also get buttons as soon as a second panel is added"
    panels size = 2 ifTrue: [panels first createMorph].

    self browser ifNotNil: [aPanel browser: self browser].
    self changed: #columns! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/8/2007 11:40'!
browser: aBrowser
    super browser: aBrowser.
    self panels do: [:ea | ea browser: aBrowser].! !

!OBMultipleDefinitionPanel methodsFor: 'building' stamp: 'dr 6/12/2007 16:11'!
buildOn: aBuilder
	^ aBuilder multipleDefinitionPanel: self with: []! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/12/2007 16:12'!
columns
   ^self panels! !

!OBMultipleDefinitionPanel methodsFor: 'callbacks' stamp: 'dr 5/8/2007 11:29'!
defaultBackgroundColor
	^ browser defaultBackgroundColor! !

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 5/29/2007 15:55'!
definitionChanged: ann
    "self panels first definition: ann definition.
    self panels first definitionChanged: ann."! !

!OBMultipleDefinitionPanel methodsFor: 'testing' stamp: 'dr 6/7/2007 11:36'!
hasChangeablePanel
    ^self panels anySatisfy: [:ea | ea isChangeable].! !

!OBMultipleDefinitionPanel methodsFor: 'testing' stamp: 'dr 5/25/2007 17:29'!
hasMethodPanel
    ^self panels anySatisfy: [:ea | ea node isKindOf: OBMethodNode].! !

!OBMultipleDefinitionPanel methodsFor: 'testing' stamp: 'dr 5/29/2007 10:40'!
includesNode: aNode
    ^self panels anySatisfy: [:ea | ea node = aNode].! !

!OBMultipleDefinitionPanel methodsFor: 'initializing' stamp: 'dr 6/7/2007 11:55'!
initialize
    panels := OrderedCollection new.    
    self addPanel: OBClosableDefinitionPanel new.
    ! !

!OBMultipleDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/14/2007 10:39'!
insert: firstPanel after: secondPanel
    | removeIdx |
    removeIdx := self panels indexOf: firstPanel.
    self panels add: firstPanel after: secondPanel.
    self panels removeAt: removeIdx.
    self changed: #columns.! !

!OBMultipleDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/14/2007 10:39'!
insert: firstPanel before: secondPanel
    | removeIdx |
    removeIdx := self panels indexOf: firstPanel.
    self panels add: firstPanel before: secondPanel.
    self panels removeAt: (removeIdx + 1).
    self changed: #columns.! !

!OBMultipleDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/11/2007 15:45'!
is: firstClosableDefinitionPanel placedBefore: secondClosableDefinitionPanel
    ^(self panels indexOf: firstClosableDefinitionPanel) < (self panels indexOf: secondClosableDefinitionPanel)! !

!OBMultipleDefinitionPanel methodsFor: 'testing' stamp: 'dr 5/24/2007 12:20'!
isDefinition
   ^true! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/7/2007 11:50'!
lastChangeablePanel "self halt."
    ^self panels reversed detect: [:ea | ea isChangeable] ifNone: [nil].! !

!OBMultipleDefinitionPanel methodsFor: 'testing' stamp: 'dr 5/31/2007 17:12'!
lastIsNonMetaPanel
    ^(self lastPanel node isKindOf: OBMethodNode) not and: [self lastPanel isKindOf: OBClosableDefinitionPanel]! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/31/2007 17:06'!
lastPanel
    ^self panels last! !

!OBMultipleDefinitionPanel methodsFor: 'constructing' stamp: 'dr 5/8/2007 11:27'!
morph
	^ (OBPaneScroller withModel: self)
		name: 'def';
		yourself! !

!OBMultipleDefinitionPanel methodsFor: 'constructing' stamp: 'dr 5/8/2007 11:32'!
morphHeight
	^ 0! !

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 5/25/2007 15:28'!
nodeDeleted: ann
    | deletedNodePanel  |
    (ann node isKindOf: OBMethodNode) ifTrue: [
	   deletedNodePanel := self panels detect: [:ea | ea node = ann node] ifNone: [nil].
	   deletedNodePanel ifNotNil: [self removePanel: deletedNodePanel column: nil]
    ].! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/24/2007 12:27'!
panels
   ^panels! !

!OBMultipleDefinitionPanel methodsFor: 'callbacks' stamp: 'dr 5/8/2007 11:26'!
panes
    ^panels collect: [:ea | ea morph]! !

!OBMultipleDefinitionPanel methodsFor: 'callbacks' stamp: 'dr 5/8/2007 11:44'!
reclaimPanes
    ^0! !

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 5/18/2007 11:40'!
refresh: ann
    self panels first refresh: ann.! !

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 5/25/2007 18:00'!
removeNonMethodPanels
	self panels copy do: [:ea | (ea node isKindOf: OBMethodNode) ifFalse: [self removePanel: ea]]! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/25/2007 17:55'!
removePanel: aPanel
    ^self removePanel: aPanel column: nil! !

!OBMultipleDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/14/2007 10:39'!
removePanel: aPanel column: aColumn
    | newPanel theClass |
    panels remove: aPanel.
 
    "make sure that the last panels does not have any buttons"
    panels size = 1 ifTrue: [panels first createMorph].
    
    "make sure that there is always at least a standard panel"
    panels ifEmpty: [ 
	    newPanel := OBClosableDefinitionPanel new. 
	    aColumn ifNotNil: [newPanel definition: aColumn selectedNode definition]
	               ifNil: [aPanel node ifNotNil: [theClass := aPanel node theClass.
			            newPanel definition: (OBMethodDefinition source: theClass sourceCodeTemplate inClass: theClass)]]. 
	    self addPanel: newPanel.
    ].
    self changed: #columns! !

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 6/29/2007 16:31'!
selectionChanged: ann
    | newPanel newDefinition deselectedPanel panel |
    self flag: #toRefactor.

	ann deselectedNode ifNotNil: [
       deselectedPanel := self panels detect: [:ea | ea node = ann deselectedNode] ifNone: [nil].
       deselectedPanel ifNotNil: [self removePanel: deselectedPanel column: ann column]
	] ifNil: [.
        (self panels noneSatisfy: [:ea | ea node = ann node]) ifTrue: [
	       newDefinition := ann node ifNotNil: [ann node definition].
		  (newDefinition notNil and: [newDefinition text notEmpty]) ifTrue: [
	           (self hasChangeablePanel and: [ActiveEvent shiftPressed not]) ifTrue: [
		             self lastChangeablePanel definition: newDefinition.
		             self lastChangeablePanel node: ann node.
		             self lastChangeablePanel changed. 
		             self changed: #columns. 		
		       ] ifFalse: [
                      newPanel := OBClosableDefinitionPanel new.
                      newPanel definition: newDefinition.
                      newPanel node: ann node.
                      self addPanel: newPanel.
                ].
             ] 
         ] ifFalse: [
                "update panel for this node with new source code"
                panel := self panels detect: [:ea | ea node = ann node].
                panel definition: ann node definition.
         ].
     ].! !

!OBMultipleDefinitionPanel methodsFor: 'callbacks' stamp: 'dr 5/29/2007 15:28'!
sizing 
    self panels ifEmpty: [^1].
    ^self panels size! !

!OBMultipleDefinitionPanel methodsFor: 'updating' stamp: 'dr 5/25/2007 14:57'!
subscribe
	self announcer 
		observe: OBAboutToChange
		send: #aboutToChange:
		to: self;
		
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self;
		
		observe: OBRefreshRequired
		send: #refresh:
		to: self;
				
		observe: OBDefinitionChanged
		send: #definitionChanged: 
		to: self;
		
		observe: OBNodeDeleted
		send: #nodeDeleted: 
		to: self.! !

!OBMultipleDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/14/2007 10:39'!
swap: firstPanel with: secondPanel
    self panels swap: (self panels indexOf: firstPanel) with: (self panels indexOf: secondPanel).
    self changed: #columns.
"self halt."! !

!OBPanel class methodsFor: 'instance creation' stamp: 'cwp 8/31/2004 10:46'!
inBrowser: aBrowser
	^ self new browser: aBrowser! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 10/13/2006 09:29'!
announce: aClass 
	^ self browser announce: aClass! !

!OBPanel methodsFor: 'accessing' stamp: 'dr 7/5/2007 11:16'!
announcer
	^self browser announcer! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:58'!
browser
	^ browser! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 4/17/2006 22:17'!
browser: aBrowser
	browser _ aBrowser.
	self subscribe! !

!OBPanel methodsFor: 'testing' stamp: 'dr 5/24/2007 12:20'!
isDefinition
   ^false! !

!OBPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:09'!
isNavigation
	^ false! !

!OBPanel methodsFor: 'testing' stamp: 'dr 6/28/2007 10:36'!
isTab
    ^false! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 4/17/2006 19:36'!
subscribe! !

OBPanel subclass: #OBTextPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

OBTextPanel subclass: #OBDefinitionPanel
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBDefinitionPanel commentStamp: 'cwp 1/7/2005 23:35' prior: 0!
OBDefinition serves as the model for the text pane of a typical browser. It's main responsibility is to act as a relay between a PluggableTextMorph and a Definition supplied by the currently selected node.

iVars:

browser - The browser of which this panel is a part.
!

OBDefinitionPanel subclass: #OBClosableDefinitionPanel
	instanceVariableNames: 'node morph isPinned owner contentMorph codeTabPanel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/8/2007 15:54'!
asDraggableMorph
	^(StringMorph contents: '*')
		color: Color white;
		yourself! !

!OBClosableDefinitionPanel methodsFor: 'building' stamp: 'dr 6/18/2007 10:36'!
buildOn: aBuilder
    ^aBuilder closableDefinitionPanel: self with: []! !

!OBClosableDefinitionPanel methodsFor: 'tabs' stamp: 'dr 6/29/2007 16:27'!
changed
    self codeTabPanel selection = 1 ifFalse: [self codeTabPanel selection: 1.].! !

!OBClosableDefinitionPanel methodsFor: 'class selecting' stamp: 'dr 5/29/2007 11:25'!
classLabel
	| classLabel |
	classLabel := self className.
	self node ifNotNil: [self node isMethodHolderNode ifTrue: [classLabel := classLabel,' >>']].
	^classLabel! !

!OBClosableDefinitionPanel methodsFor: 'class selecting' stamp: 'dr 5/29/2007 11:25'!
className
     ^self methodClass ifNotNil: [self methodClass name] ifNil: ['']! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 5/29/2007 11:26'!
close
   browser definitionPanel panels size < 2 ifTrue: [^self inform: 'You cannot close the last panel.'].
   browser definitionPanel removePanel: self column: nil.! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 5/29/2007 11:26'!
closeLabel
	^ (StringMorph 
		contents: 'x'
		font: ("TextStyle defaultFont" StrikeFontSet familyName: 'Accouny14' size: 10))
			color: Color red;
			yourself! !

!OBClosableDefinitionPanel methodsFor: 'building' stamp: 'dr 6/29/2007 15:22'!
codeTabPanel
    codeTabPanel ifNil: [codeTabPanel := (OBCodeTabPanel new browser: browser; definitionPanel: self; yourself)].
    ^codeTabPanel! !

!OBClosableDefinitionPanel methodsFor: 'tabs' stamp: 'dr 6/22/2007 16:49'!
color
	^browser defaultBackgroundColor ! !

!OBClosableDefinitionPanel methodsFor: 'tabs' stamp: 'dr 6/22/2007 16:45'!
commands
	| myNode |
	myNode _ self currentNode.
	myNode ifNil: [^ #()].
     ^{OBCmdSystemComplexity on: myNode for: self. OBCmdUmlDiagram on: myNode for: self}
    ! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 7/12/2007 13:55'!
createClassLabel
	| classLabel |		
     classLabel := (StringMorph 
		contents: (Text string: self classLabel attribute: TextEmphasis bold)	
		font: (StrikeFontSet familyName: #NewYork size: 10))
			color: Color black;
			yourself.
	classLabel on: #mouseDown send: #selectClass to: self.			
	classLabel hResizing: #spaceFill; vResizing: #spaceFill; setWidth: 100; height: 20.						
			
	^classLabel		! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/5/2007 17:45'!
createCloseButton						
	^PluggableButtonMorph
		on: self
		getState: #isSelected
		action: #close
		label: #closeLabel.! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/29/2007 16:07'!
createContentMorph
	| textEditor |
	
	textEditor := (MorphicTextEditor default
		on: self
		text: #text
		accept: #accept:notifying:
		readSelection: #selection
		menu: #menu:shifted:)
		font: Preferences standardCodeFont;
		yourself.
	textEditor 
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		color: Color white.
     ^textEditor! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/12/2007 15:20'!
createDragButton
	| dragButton paneColor |
	dragButton := OBDraggableButtonMorph 
				on: self
				getState: #isDragged
				action: #drag
				label: #dragLabel.
	dragButton enableDragNDrop.
	paneColor := browser defaultBackgroundColor.
	dragButton onColor: Color lightGray lighter offColor: paneColor.
	^ dragButton! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 7/12/2007 14:59'!
createMorph					
	morph := (RectangleMorph new)
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill;		
		wrapCentering: #center;
		cellPositioning: #topLeft;
		rubberBandCells: true;
		layoutInset: 2;
		cellInset: 2;
		color: Color white;
		yourself.

	morph addMorphBack: self codeTabPanel morph.		
	browser definitionPanel panels size > 1 ifTrue: [morph addMorphBack: self createTitleBar].
	contentMorph := self createContentMorph.		
	morph addMorphBack: contentMorph.	
	self codeTabPanel selection = 1 ifFalse: [self codeTabPanel executeCommand].		
			
	^morph		! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/12/2007 15:20'!
createPinButton
	| pinButton paneColor |
	pinButton := OBDraggableButtonMorph 
				on: self
				getState: #isPinned
				action: #pin
				label: #pinLabel.
	paneColor := browser defaultBackgroundColor.
	pinButton onColor: Color lightGray lighter offColor: paneColor.
	^ pinButton! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/11/2007 11:48'!
createTitleBar
	| titleBar |		
	titleBar := (RectangleMorph new)
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		borderWidth: 0;
		"borderColor: Color black;"
		wrapCentering: #center;
		cellPositioning: #topLeft;
		rubberBandCells: true;
		layoutInset: 2;
		cellInset: 2;
		color: Color white;
		yourself.
			
	titleBar addMorphBack: self createClassLabel; 
	          addMorphBack: self createDragButton; 
	          addMorphBack: self createPinButton; 
	          addMorphBack: self createCloseButton.			
	^titleBar		! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 15:50'!
currentNode
    ^self node ifNil: [super currentNode]! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 6/11/2007 11:47'!
drag
    "do nothing"! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 6/11/2007 11:49'!
dragLabel
	^ (StringMorph 
		contents: '<- ->'
		font: (StrikeFontSet familyName: 'Accouny14' size: 10))
			color: Color red;
			yourself! !

!OBClosableDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/11/2007 15:43'!
dropMorph: aMorph
    | panelToBeMoved |
    panelToBeMoved := aMorph passenger.

    (self isPlacedBefore: panelToBeMoved) ifTrue: [
	   panelToBeMoved insertBefore: self. 
    ] ifFalse: [
        panelToBeMoved insertAfter: self.
    ].! !

!OBClosableDefinitionPanel methodsFor: 'tabs' stamp: 'dr 6/29/2007 15:34'!
exchangeContentWith: aMorph
    morph removeMorph: contentMorph. 
    aMorph vResizing: #spaceFill.
    contentMorph := aMorph. 
    morph addMorphBack: contentMorph.! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 15:59'!
getDefinition
     definition ifNotNil: [^definition].
	^ browser currentNode ifNotNilDo: [:ea | ea definition]! !

!OBClosableDefinitionPanel methodsFor: 'initializing' stamp: 'dr 6/7/2007 11:20'!
initialize
    isPinned := false! !

!OBClosableDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/8/2007 16:33'!
insertAfter: aPanel
    ^self owner insert: self after: aPanel! !

!OBClosableDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/11/2007 15:44'!
insertBefore: aPanel
    ^self owner insert: self before: aPanel! !

!OBClosableDefinitionPanel methodsFor: 'testing' stamp: 'dr 6/7/2007 11:36'!
isChangeable
    ^self isPinned not! !

!OBClosableDefinitionPanel methodsFor: 'testing' stamp: 'dr 6/11/2007 11:47'!
isDragged
    ^false! !

!OBClosableDefinitionPanel methodsFor: 'testing' stamp: 'dr 6/6/2007 15:59'!
isPinned
    ^isPinned! !

!OBClosableDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/11/2007 15:35'!
isPlacedBefore: aClosableDefinitionPanel
    ^self owner is: self placedBefore: aClosableDefinitionPanel! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 5/29/2007 11:26'!
isSelected
    ^false
   ! !

!OBClosableDefinitionPanel methodsFor: 'class selecting' stamp: 'dr 5/29/2007 11:25'!
methodClass
     ^(self node respondsTo: #theClass) ifTrue: [self node theClass] ifFalse: [nil]! !

!OBClosableDefinitionPanel methodsFor: 'morphic' stamp: 'dr 6/29/2007 16:06'!
morph
	morph ifNotNil: [^morph].
	^self createMorph! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 11:45'!
node
    ^node! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/29/2007 15:15'!
node: aNode
    node := aNode. 
    "self selectSourceTab."! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/8/2007 16:16'!
owner
	^owner! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/8/2007 16:16'!
owner: aMultipleDefinitionPanel
	owner := aMultipleDefinitionPanel! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 6/7/2007 11:50'!
pin
    self isPinned ifTrue: [isPinned := false] ifFalse: [isPinned := true] ! !

!OBClosableDefinitionPanel methodsFor: 'buttons' stamp: 'dr 6/6/2007 15:20'!
pinLabel
	^ (StringMorph 
		contents: '!!'
		font: (StrikeFontSet familyName: 'Accouny14' size: 10))
			color: Color red;
			yourself! !

!OBClosableDefinitionPanel methodsFor: 'class selecting' stamp: 'dr 5/29/2007 11:25'!
selectClass
    browser jumpTo: (OBClassNode on: self methodClass)! !

!OBClosableDefinitionPanel methodsFor: 'tabs' stamp: 'dr 6/29/2007 16:31'!
selectSourceTab
    self exchangeContentWith: self createContentMorph.! !

!OBClosableDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/6/2007 15:59'!
setIsPinned
    isPinned := true! !

!OBClosableDefinitionPanel methodsFor: 'reorganizing' stamp: 'dr 6/8/2007 16:19'!
swapWith: aPanel
    ^self owner swap: self with: aPanel! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'PDC 6/27/2007 07:28'!
aboutToChange: ann 
	self canDiscardEdits ifTrue: [^ self].
	(OBConfirmationRequest 
		prompt: 'Changes have not been saved.
Is it OK to discard those changes?'
		confirm: 'Discard changes') 
			ifTrue: [self changed: #clearUserEdits]
			ifFalse: [ann veto]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 3/3/2007 23:11'!
aboutToChangeSilently: ann
	self canDiscardEdits ifFalse: [ann veto]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 6/4/2006 00:41'!
accept: aText notifying: aController
	^ self 
		withDefinitionDo: [:def | [def accept: aText notifying: aController]
									on: OBAnnouncerRequest
									do: [:notification | notification resume: self announcer]]
		ifNil: [true]! !

!OBDefinitionPanel methodsFor: 'building' stamp: 'cwp 3/16/2007 23:09'!
buildOn: aBuilder
	^ aBuilder definitionPanel: self with: []! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 15:39'!
currentNode
	^browser currentNode! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 16:00'!
definition: aDefinition
	definition _ aDefinition.
	self changed: #text! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 6/4/2006 11:31'!
definitionChanged: ann
	definition _ ann definition.
	self changed: #text! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItContext
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItContext) ifTrue: [def doItContext]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItReceiver
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItReceiver) ifTrue: [def doItReceiver]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 4/19/2007 21:56'!
environment
	^ self selectedClass environment! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'dr 6/28/2007 11:17'!
getDefinition
	^ browser currentNode ifNotNilDo: [:ea | ea definition]! !

!OBDefinitionPanel methodsFor: 'testing' stamp: 'dr 5/29/2007 11:16'!
isDefinition
    ^true! !

!OBDefinitionPanel methodsFor: 'user interface' stamp: 'dr 5/29/2007 16:35'!
menu: aMenu shifted: shifted
    ^nil! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'dr 5/29/2007 15:39'!
node
	^nil! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 6/3/2006 21:55'!
refresh: announcement
	| oldDefinition |
	definition ifNil: [^ self].
	oldDefinition _ definition.
	definition _ self getDefinition.
	definition text = oldDefinition text ifTrue: [^ self].
	self canDiscardEdits
			ifTrue: [self changed: #text]
			ifFalse: [self changed: #codeChangedElsewhere]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:22'!
selectedClass
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #selectedClass) ifTrue: [def selectedClass]] 
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 21:15'!
selection
	^ self withDefinitionDo: [:def | def textSelection] ifNil: [1 to: 0]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 4/24/2006 16:06'!
selectionChanged: ann
	self definition: self getDefinition! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 3/3/2007 22:49'!
subscribe
	self announcer 
		observe: OBAboutToChange
		send: #aboutToChange:
		to: self;
		
		observe: OBAboutToChangeSilently
		send: #aboutToChangeSilently:
		to: self;
		
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self;
		
		observe: OBRefreshRequired
		send: #refresh:
		to: self;
				
		observe: OBDefinitionChanged
		send: #definitionChanged: 
		to: self.! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'dr 3/6/2007 16:37'!
text 
	^self withDefinitionDo: [:def | def text] ifNil: [''].

	! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'dc 6/1/2007 12:10'!
vResizing
	^ #spaceFill! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:41'!
withDefinitionDo: workBlock ifNil: nilBlock
	definition ifNil: [ ^ nilBlock value].
	^ workBlock value: definition! !

OBDefinitionPanel subclass: #OBMondrianPanel
	instanceVariableNames: 'visualization contentMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 7/3/2007 14:58'!
classBlueprint
    self setVisualization: #classBlueprint.
    ^self morph! !

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 7/12/2007 13:30'!
createClassBlueprint
	| view theClass |
	theClass := self currentNode theClass.
	view := MOClassBlueprint ofClass: theClass inPanel: self.
	^self createContentMorphWithView: view! !

!OBMondrianPanel methodsFor: 'morphic' stamp: 'dr 6/22/2007 14:41'!
createContentMorph
    ^self perform: ('create', visualization asString capitalized) asSymbol
! !

!OBMondrianPanel methodsFor: 'morphic' stamp: 'dr 6/22/2007 15:00'!
createContentMorphWithView: aView
    contentMorph := ScrollPane new
          borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill.

    contentMorph scroller addMorph: (MOViewer on: aView graph).
    ^contentMorph
! !

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 7/3/2007 10:37'!
createSystemComplexity
    ^self createContentMorphWithView: (MOPainting createSystemComplexityOf: self currentNode theClass)! !

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 7/3/2007 10:57'!
createUmlDiagram
	| view theClass |
	theClass := self currentNode theClass.
	view := MORenderer new.
	view layout: ((MOVerticalSpacedTreeLayout new)
				gap: [:ea | 10 @ 40];
				roots: [:each | Array with: (view elementOf: theClass)];
				children: [:each | each subclasses collect: [:ea | view elementOf: ea]]).
	view style: MOUmlClass new.
	view addAll: theClass withAllSubclasses.
	view layout: ((MOConnectorLayout new)
				source: [:each | view graph elementOf: each model key];
				sourcePosition: #topCenter;
				target: [:each | view graph elementOf: each model value];
				targetPosition: #bottomCenter).
	view style: MOUmlInheritance new.
	view 
		addAll: (theClass allSubclasses collect: [:each | each -> each superclass]).
	^ self createContentMorphWithView: view! !

!OBMondrianPanel methodsFor: 'initializing' stamp: 'dr 6/22/2007 15:00'!
initialize
    super initialize.
    visualization := #systemComplexity! !

!OBMondrianPanel methodsFor: 'morphic' stamp: 'dr 6/29/2007 14:40'!
morph
    ^self createContentMorph! !

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 6/22/2007 14:38'!
setVisualization: aSymbol
    visualization := aSymbol! !

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 6/29/2007 14:46'!
systemComplexity
    self setVisualization: #systemComplexity.
    ^self morph! !

!OBMondrianPanel methodsFor: 'visualizations' stamp: 'dr 6/29/2007 14:46'!
umlDiagram
	self setVisualization: #umlDiagram.
	^self morph! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'PDC 6/26/2007 22:37'!
accept: aString notifying: aController
	self subclassResponsibility ! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 21:35'!
addEditingItemsToMenu: aMenu shifted: aBoolean
	| items |
	items _  aBoolean 
		ifTrue: [self shiftedYellowButtonMenu] 
		ifFalse: [self yellowButtonMenu].
	items do: [:ea |
			  ea = #-
				ifFalse: [aMenu add: ea first action: ea second]
				ifTrue: [aMenu addLine]].
! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 21:35'!
addMoreToMenu: aMenu
	aMenu addLine.
	aMenu
		add: 'more...' translated
		action: #shiftedTextPaneMenuRequest! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 22:08'!
addTextCommandsToMenu: aMenu selection: aTextSelection
	| scan |
	scan _ self announce: OBTextCommandScan.
	scan
		populateMenu: aMenu
		withNode: aTextSelection
		forRequestor: self.
! !

!OBTextPanel methodsFor: 'building' stamp: 'PDC 6/26/2007 22:37'!
buildOn: aBuilder
	self subclassResponsibility ! !

!OBTextPanel methodsFor: 'accessing' stamp: 'PDC 6/26/2007 22:55'!
environment
	^ Smalltalk! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 4/19/2007 22:08'!
menu: aMenu shifted: aBoolean selection: aTextSelection 
	
	self addEditingItemsToMenu: aMenu shifted: aBoolean.
	self addTextCommandsToMenu: aMenu selection: aTextSelection.
	self addMoreToMenu: aMenu.
	^ aMenu ! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 4/19/2007 21:32'!
perform: aSelector orSendTo: anObject
	| receiver |
	receiver  _ (self respondsTo: aSelector) ifTrue: [self] ifFalse: [anObject].
	receiver perform: aSelector! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'PDC 6/26/2007 22:39'!
selection
	self subclassResponsibility ! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 7/1/2007 16:56'!
shiftedYellowButtonMenu
	^ Array streamContents:
		[:stream |
		stream
			nextPut: (Array with: 'explain' translated with: #explain);
			nextPut: #-;
			nextPut: (Array 
						with: 'selectors containing it (W)' translated 
						with: #methodNamesContainingIt);
			nextPut: (Array 
						with: 'method strings with it (E)' translated 
						with: #methodStringsContainingit);
			nextPut: (Array 
						with: 'method source with it' translated 
						with: #methodSourceContainingIt);
			nextPut: (Array 
						with: 'class names containing it' translated 
						with: #classNamesContainingIt);
			nextPut: (Array 
						with: 'class comments with it' translated 
						with: #classCommentsContainingIt);
			nextPut: (Array 
						with: 'change sets with it' translated 
						with:  #browseChangeSetsWithSelector);
			nextPut: #-;
			nextPut: (Array with: 'save contents to file...' translated with: #saveContentsInFile);
			nextPut: #-;
			nextPut: (Array with: 'more...' translated with: #yellowButtonActivity)]
! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'PDC 6/26/2007 22:38'!
text
	self subclassResponsibility ! !

!OBTextPanel methodsFor: 'building' stamp: 'PDC 6/26/2007 22:38'!
vResizing
	self subclassResponsibility ! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 7/1/2007 16:51'!
yellowButtonMenu
	^ Array streamContents:
		[:stream |
		stream 	
			nextPut: (Array with: 'find...(f)' translated with: #find);
			nextPut: (Array with: 'find again (g)' translated with: #findAgain);
			nextPut: (Array with: 'set search string (h)' translated with: #setSearchString);
			nextPut: #-;
			nextPut: (Array with: 'do again (j)' translated with: #again);
			nextPut: (Array with: 'undo (z)' translated with: #undo);
			nextPut: #-;
			nextPut: (Array with: 'copy (c)' translated with: #copySelection);
			nextPut: (Array with: 'cut (x)' translated with: #cut);
			nextPut: (Array with: 'paste (v)' translated with: #paste);
			nextPut: (Array with: 'paste...' translated with: #pasteRecent);
			nextPut: #-;
			nextPut: (Array with: 'accept (s)' translated with: #accept);
			nextPut: (Array with: 'cancel (l)' translated with: #cancel)]! !

Object subclass: #OBPlatform
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 12:28'!
addPreference: aSymbol categories: anArray default: aBoolean balloonHelp: aString 

	^ self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 12:20'!
annotationPanes

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'menu icons' stamp: 'dkh 6/4/2007 11:42'!
blankMenuIcon

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 12:25'!
chasingBrowsers

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/4/2007 16:37'!
classNamed: className

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/13/2007 16:47'!
classNames

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'as yet unclassified' stamp: 'cwp 6/1/2007 16:08'!
current
	^ Current ifNil: [self default]! !

!OBPlatform class methodsFor: 'as yet unclassified' stamp: 'cwp 6/1/2007 15:46'!
current: aPlatform
	Current _ aPlatform! !

!OBPlatform class methodsFor: 'as yet unclassified' stamp: 'cwp 6/1/2007 16:10'!
default
	| classes |
	classes _ self subclasses.
	^ classes isEmpty
		ifTrue: [self error: 'No platform available']
		ifFalse: [classes anyOne default]! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 11:31'!
defaultAnnotationRequests

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/4/2007 12:38'!
enableGently: attribute

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/4/2007 16:32'!
hasClassNamed: aSymbol

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 13:22'!
lexiconWindowColor

	^ self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/4/2007 11:39'!
menuWithIcons

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 12:29'!
mercuryPanel

	^ self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/11/2007 13:19'!
messageListWindowColor

	^ self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/4/2007 11:38'!
optionalButtons

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'preferences' stamp: 'dkh 6/4/2007 11:40'!
setPreference: attribute toValue: value

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/4/2007 16:47'!
smalltalkAssociationAt: aSymbol

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/4/2007 16:32'!
smalltalkAt: aSymbol

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/4/2007 16:40'!
smalltalkAt: aGlobalName ifAbsent: aBlock 

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/4/2007 16:42'!
smalltalkAt: aGlobalName ifPresent: aBlock 

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'smalltalk' stamp: 'dkh 6/5/2007 11:03'!
smalltalkEnvironment

	^self subclassResponsibility! !

!OBPlatform class methodsFor: 'menu icons' stamp: 'dkh 6/5/2007 14:57'!
tryIcons: anArray

	^self subclassResponsibility! !

Object subclass: #OBSwitch
	instanceVariableNames: 'column filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBSwitch class methodsFor: 'instance creation' stamp: 'cwp 5/17/2007 23:47'!
inColumn: aColumn 
	^ self basicNew setColumn: aColumn filter: nil.! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/21/2007 22:25'!
buildOn: aBuilder
	^ aBuilder switch: self with: []! !

!OBSwitch methodsFor: 'accessing' stamp: 'cwp 5/18/2007 00:06'!
filter
	^ filter! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/17/2007 22:59'!
filter: aFilter
	filter _ aFilter.
	filter ifNotNil:
		[filter activate.
		self changed: #list]! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/17/2007 21:41'!
isActive
	^ filter notNil! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/17/2007 22:26'!
list
	^ filter 
		ifNotNil: [filter list]
		ifNil: [#()]! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/18/2007 21:45'!
refresh
	self changed: #selection! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/17/2007 22:12'!
selection
	^ filter 
		ifNotNil: [filter selection]
		ifNil: [0]
! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/17/2007 23:48'!
selection: anInteger
	filter ifNotNil: 
		[filter selection: anInteger.
		self changed: #selection.
		column listChanged]! !

!OBSwitch methodsFor: 'as yet unclassified' stamp: 'cwp 5/17/2007 23:44'!
setColumn: aColumn filter: aFilter
	column _ aColumn.
	filter _ aFilter.! !

Object subclass: #OBTextSelection
	instanceVariableNames: 'selection text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBTextSelection class methodsFor: 'instance creation' stamp: 'cwp 10/13/2006 16:52'!
on: anOBPluggableTextMorph 
	^ self new! !

!OBTextSelection class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 20:10'!
on: anInterval inText: aString
	^ self new setSelection: anInterval inText: aString! !

!OBTextSelection class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 20:59'!
onAllOf: aString 
	^ self on: (1 to: aString size) inText: aString! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:15'!
fullText
	^ text! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:44'!
hasSelector
	^ self selector notNil! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 21:31'!
selector
	^ self text asString findSelector! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:12'!
setSelection: anInterval inText: aString
	selection _ anInterval.
	text _ aString.! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:14'!
text
	^ text atAll: selection! !

!Object methodsFor: '*omnibrowser-converting' stamp: 'cwp 4/17/2006 12:16'!
asAnnouncement
	^ self! !

!Behavior methodsFor: '*omnibrowser-converting' stamp: 'cwp 4/17/2006 12:16'!
asAnnouncement
	^ self new! !
